<?php
/** Classe AbulEduLOMV1
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/Format_LOM_LOMFR_ScoLOMFR
  * @see http://www.lom-fr.fr/lomfr-unt/unisciel/modeles-de-fiche/LOMFRTemplate-Unisciel-v1.0-complet.xml/view
  *
  * @author 2011-2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @author 2012 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program. If not, see <http://www.gnu.org/licenses/>.
  */

  //error_reporting(E_ALL);

include_once 'LomLS.php';
include_once 'VCard/IMC.php';
class AbulEduLOMV1{
    
    /**
      * Initialisation des vocabulaire du Lom 
      */
    public function __construct(){
        //ABULEDU_LOG_TRACE
        $this->m_xmlPrefixLom = "lom:"; /** Pour prefixer tous les objets XML */

        //Valeur par defaut abuledu
        $this->m_GeneralIdentifierCatalog = "URI";
        
        //Initialisation de la liste des langues
        $this->m_availableLanguages = array();
        $this->m_availableLanguages[""] = "Non Enregistrée";
        $this->m_availableLanguages["fre"] = "Français";
        $this->m_availableLanguages["eng"] = "Anglais";
        $this->m_availableLanguages["spa"] = "Espagnol";
        $this->m_availableLanguages["ita"] = "Italien";
        $this->m_availableLanguages["ger"] = "Allemand";
        $this->m_availableLanguages["ara"] = "Arabe";
        $this->m_availableLanguages["dut"] = "Néerlandais";
        $this->m_availableLanguages["oci"] = "Occitan";

        //Initialisation des listes du vocabublaire obligatoire
        //Vocabulaire de la structure 1.7
        $this->m_GeneralStructureVocabLOM = array();
        $this->m_GeneralStructureVocabLOM["atomic"] = "atomique";
        $this->m_GeneralStructureVocabLOM["collection"] = "collection";
        $this->m_GeneralStructureVocabLOM["hierarchical"] = "hiérarchique";
        $this->m_GeneralStructureVocabLOM["linear"] = "linéaire";
        $this->m_GeneralStructureVocabLOM["networked"] = "en réseau";

        //structure 1.8
        $this->m_GeneralAggregationLevelVocabLOM = array();
        $this->m_GeneralAggregationLevelVocabLOM["1"] = "1";
        $this->m_GeneralAggregationLevelVocabLOM["2"] = "2";
        $this->m_GeneralAggregationLevelVocabLOM["3"] = "3";
        $this->m_GeneralAggregationLevelVocabLOM["4"] = "4";

        //structure 2.2
        $this->m_LifeCycleStatusVocabLOM = array();
        $this->m_LifeCycleStatusVocabLOM["draft"] = "brouillon";
        $this->m_LifeCycleStatusVocabLOM["final"] = "définitive";
        $this->m_LifeCycleStatusVocabLOM["revised"] = "relue";
        $this->m_LifeCycleStatusVocabLOM["unvailable"] = "inconnue";

        //structure 2.3.1
        $this->m_LifeCycleContributionRoleVocabLOM = array();
        $this->m_LifeCycleContributionRoleVocabLOM["author"] = "auteur";
        $this->m_LifeCycleContributionRoleVocabLOM["publisher"] = "diffuseur";
        $this->m_LifeCycleContributionRoleVocabLOM["unknown"] = "inconnu";
        $this->m_LifeCycleContributionRoleVocabLOM["initiator"] = "créateur";
        $this->m_LifeCycleContributionRoleVocabLOM["terminator"] = "modérateur";
        $this->m_LifeCycleContributionRoleVocabLOM["validator"] = "contrôleur";
        $this->m_LifeCycleContributionRoleVocabLOM["editor"] = "éditeur";
        $this->m_LifeCycleContributionRoleVocabLOM["graphical designer"] = "concepteur graphique";
        $this->m_LifeCycleContributionRoleVocabLOM["technical implementer"] = "développeur";
        $this->m_LifeCycleContributionRoleVocabLOM["content provider"] = "fournisseur de contenu";
        $this->m_LifeCycleContributionRoleVocabLOM["technical validator"] = "contrôleur technique";
        $this->m_LifeCycleContributionRoleVocabLOM["educational validator"] = "contrôleur pédagogique";
        $this->m_LifeCycleContributionRoleVocabLOM["script writer"] = "rédacteur";
        $this->m_LifeCycleContributionRoleVocabLOM["instructional designer"] = "concepteur didactique";
        $this->m_LifeCycleContributionRoleVocabLOM["subject matter expert"] = "expert conseil";
        
        //structure 5.1
        $this->m_EducationalInteractivityTypeVocabLOM = array();
        $this->m_EducationalInteractivityTypeVocabLOM["active"] = "actif";
        $this->m_EducationalInteractivityTypeVocabLOM["expositive"] = "explicatif";
        $this->m_EducationalInteractivityTypeVocabLOM["mixed"] = "mixte";

        //5.2
        $this->m_EducationalLearningResourceTypeVocabLOM = array();
        $this->m_EducationalLearningResourceTypeVocabLOM["exercise"] = "exercice";
        $this->m_EducationalLearningResourceTypeVocabLOM["simulation"] = "simulation";
        $this->m_EducationalLearningResourceTypeVocabLOM["questionnaire"] = "questionnaire";
        $this->m_EducationalLearningResourceTypeVocabLOM["diagram"] = "diagramme";
        $this->m_EducationalLearningResourceTypeVocabLOM["figure"] = "figure";
        $this->m_EducationalLearningResourceTypeVocabLOM["graph"] = "graphique";
        $this->m_EducationalLearningResourceTypeVocabLOM["index"] = "index";
        $this->m_EducationalLearningResourceTypeVocabLOM["slide"] = "diapositive";
        $this->m_EducationalLearningResourceTypeVocabLOM["table"] = "tableau";
        $this->m_EducationalLearningResourceTypeVocabLOM["narrative text"] = "texte narratif";
        $this->m_EducationalLearningResourceTypeVocabLOM["exam"] = "examen";
        $this->m_EducationalLearningResourceTypeVocabLOM["experiment"] = "expérience";
        $this->m_EducationalLearningResourceTypeVocabLOM["problem statement"] = "énoncé";
        $this->m_EducationalLearningResourceTypeVocabLOM["self assessment"] = "auto-évaluation";
        $this->m_EducationalLearningResourceTypeVocabLOM["lecture"] = "lecture";
        
        //structure 5.5
        $this->m_EducationalIntendedEndUserRoleVocabLOM = array();
        $this->m_EducationalIntendedEndUserRoleVocabLOM["teacher"] = "enseignant";
        $this->m_EducationalIntendedEndUserRoleVocabLOM["author"] = "auteur";
        $this->m_EducationalIntendedEndUserRoleVocabLOM["learner"] = "apprenant";
        $this->m_EducationalIntendedEndUserRoleVocabLOM["manager"] = "directeur";

        //5.6
        $this->m_EducationalContextVocabLOM = array();
        $this->m_EducationalContextVocabLOM["school"] = "école";
        $this->m_EducationalContextVocabLOM["higher education"] = "enseignement supérieur";
        $this->m_EducationalContextVocabLOM["training"] = "entraînement";
        $this->m_EducationalContextVocabLOM["other"] = "autre";

        //5.8
        $this->m_EducationalDifficultyVocabLOM = array();
        $this->m_EducationalDifficultyVocabLOM["very easy"] = "très facile";
        $this->m_EducationalDifficultyVocabLOM["easy"] = "facile";
        $this->m_EducationalDifficultyVocabLOM["medium"] = "moyen";
        $this->m_EducationalDifficultyVocabLOM["difficult"] = "difficile";
        $this->m_EducationalDifficultyVocabLOM["very difficult"] = "très difficile";

        //6.1
        $this->m_RightsCostVocabLOM = array();
        $this->m_RightsCostVocabLOM[""] = "Non renseigné";
        $this->m_RightsCostVocabLOM["yes"] = "oui";
        $this->m_RightsCostVocabLOM["no"] = "non";

        //6.2
        $this->m_RightsCopyrightAndOtherRestrictionsVocabLOM = array();
        $this->m_RightsCopyrightAndOtherRestrictionsVocabLOM[""] = "Non renseigné";
        $this->m_RightsCopyrightAndOtherRestrictionsVocabLOM["yes"] = "oui";
        $this->m_RightsCopyrightAndOtherRestrictionsVocabLOM["no"] = "non";

        //7.1
        $this->m_Relation = array();
        $this->m_RelationKindVocabLOM = array();
        $this->m_RelationKindVocabLOM["ispartof"] = "fait partie de";
        $this->m_RelationKindVocabLOM["haspart"] = "a des parties";
        $this->m_RelationKindVocabLOM["isversionof"] = "est une version de";
        $this->m_RelationKindVocabLOM["hasversion"] = "est versionné";
        $this->m_RelationKindVocabLOM["isformatof"] = "est le format de";
        $this->m_RelationKindVocabLOM["hasformat"] = "a un format";
        $this->m_RelationKindVocabLOM["references"] = "référence";
        $this->m_RelationKindVocabLOM["isreferencedby"] = "est référencé par";
        $this->m_RelationKindVocabLOM["isbasedon"] = "est fondé sur";
        $this->m_RelationKindVocabLOM["isbasisfor"] = "est la base de";
        $this->m_RelationKindVocabLOM["requires"] = "nécessite";
        $this->m_RelationKindVocabLOM["isrequiredby"] = "est nécessaire pour";

        //structure 9.1 Purpose

        /* Présents, avec intitulé correct */

        $this->m_ClassificationPurposeVocabLOM = array();
        $this->m_ClassificationPurposeVocabLOM["prerequisite"] = "pré-requis";
        $this->m_ClassificationPurposeVocabLOM["label"] = "label";//A Supprimer
        $this->m_ClassificationPurposeVocabLOM["educational objective"] = "objectif pédagogique";
        $this->m_ClassificationPurposeVocabLOM["skill level"] = "niveau de compétence";
        $this->m_ClassificationPurposeVocabLOM["security level"] = "niveau de sécurité";
        $this->m_ClassificationPurposeVocabLOM["competency"] = "compétence";
        $this->m_ClassificationPurposeVocabLOM["discipline"] = "champ disciplinaire";
        $this->m_ClassificationPurposeVocabLOM["idea"] = "notion";
        $this->m_ClassificationPurposeVocabLOM["accessibility restrictions"] = "restriction d'accessibilité";
        $this->m_ClassificationPurposeVocabLOM["educational level"] = "niveau éducatif détaillé";

        //Initialisation des variables
        $this->m_EducationalTypicalLearningTime = new DateTime('0-0-0'); //A revoir 
        $this->m_AnnotationDate = new DateTime('0-0-0'); //A Revoir
        $this->m_AnnotationDate->setDate(0,1,1);

        $this->m_GeneralTitle = new LomLS();
        $this->m_GeneralDescription = new LomLS(); 
        $this->m_GeneralKeyword = new LomLS(); 
        $this->m_GeneralCoverage = new LomLS();
        $this->m_LifeCycleVersion = new LomLS(); 
        $this->m_TechnicalInstallationRemarks = new LomLS(); 
        $this->m_TechnicalOtherPlatformRequirements = new LomLS(); 
        $this->m_EducationalTypicalAgeRange = new LomLS();
        $this->m_EducationalDescription = new LomLS();
        $this->m_RightsDescription = new LomLS();
        $this->m_RelationResourceDescription = new LomLS();
        $this->m_AnnotationDescription = new LomLS(); 
        $this->m_ClassificationDescription = new LomLS(); 
        $this->m_ClassificationKeyword = new LomLS();

        $this->m_TechnicalLocation = array();
        $this->m_EducationalLanguage = array();
        $this->m_RelationResourceIdentifierEntry = array();
        $this->m_ClassificationTaxonPathSource = array();
        $this->m_classificationPurposes = array();
        $this->m_LifeCycleContributions = array();
        $this->m_TechnicalFormat = array();
        $this->m_EducationalLearningResourceType = array();
        $this->m_EducationalIntendedEndUserRole = array();
        $this->m_EducationalContext = array();
        $this->m_GeneralLanguage = array();
        $this->m_domDocument = new DomDocument();
        $this->m_allVcards = array();
        $this->m_Annotation = array();
        $this->m_MetaMetadata = array();
        $this->m_MetaMetadataContribute = array();
        $this->m_MetaMetadataMetadataSchema = array();
    }

    public function __destruct(){
        //ABULEDU_LOG_TRACE
    }

    /** Permet de faire la correspondance entre les langues et la traduction de leur nom
      * @return retourne une QMap dont la clef est le code d'une langue selon la norme ISO 639-2 et la valeur la traduction de son nom
      * @see http://en.wikipedia.org/wiki/List_of_ISO_639-2_codes
      */
    function abeLOMgetAvailableLanguages(){
        //ABULEDU_LOG_TRACE
        return $this->m_availableLanguages;
    }

    /**
      * 1.1.1: Le Catalogue
      * @attention cardinalité 1
      * @param value "URI" par défaut, mais peut-être modifié, aurait été un ARK si le système n'était pas trop centralisé à mon goût
      * @see http://www.bnf.fr/fr/professionnels/anx_isbn_issn_autres/a.faq_ark.html
      * @see Dublin Core  DC.Identifier
      * @note On utilisera le mot clé URI (ark:/ serait bien mais centralisé)
      * @note Exemple de code XML qui sera généré:
      * @code
       * <lom:identifier>
        *  <lom:catalog>URI</lom:catalog>
       * <lom:entry>http://mediatheque.abuledu.org/URI/5343xdfg32</lom:entry>
        *</lom:identifier>
      * @endcode
      */

    function abeLOMsetGeneralIdentifierCatalog($value)
    {
        //ABULEDU_LOG_TRACE
        $this->m_GeneralIdentifierCatalog=$value;
    }

    function abeLOMgetGeneralIdentifierCatalog()
    {
        //ABULEDU_LOG_TRACE
        return $this->m_GeneralIdentifierCatalog;
    }

    /**
      * 1.1.2: L'entrée dans le catalogue
      * @attention cardinalité 1
      * @param value code fourni par le serveur de stockage (la médiatheque dans notre cas)
      * @see abeLOMsetGeneralIdentifierCatalog()
      * @note On utlisera http://mediatheque.abuledu.org/URI/code
      */
    function abeLOMsetGeneralIdentifierEntry($value)
    {
        //ABULEDU_LOG_TRACE
        $this->m_GeneralIdentifierEntry=$value;
    }
    function abeLOMgetGeneralIdentifierEntry()
    {
        //ABULEDU_LOG_TRACE
        return $this->m_GeneralIdentifierEntry;
    }

    /**
      * 1.2: Titre
      * @attention cardinalité 1
      * @see Dublin Core  DC.Title
      * @param lang langue du titre
      * @param value le titre du document
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:title>
  <lom:string language="fre">Le titre du document en français</lom:string>
</lom:title>
      * @endcode
      */

    function abeLOMsetGeneralTitle($lang,$value)
    {
        //ABULEDU_LOG_TRACE(lang,value)
        if(array_key_exists($lang, $this->m_availableLanguages)){
            $this->m_GeneralTitle->set($lang, $value);
            return true;
        }else{
            if(in_array($lang, $this->m_availableLanguages)){
                $this->m_GeneralTitle->set(array_search($lang, $this->m_availableLanguages), $value);
                return true;
            }
        }
    }

    function abeLOMgetGeneralTitle($lang)
    {
        //ABULEDU_LOG_TRACE
        if(in_array($lang, $this->m_GeneralTitle->keys())){       
            return $this->m_GeneralTitle->get($lang);
        }

        if(array_search($lang, $this->m_availableLanguages)){

            
            $pivot = array_search($lang, $this->m_availableLanguages);
            if(array_search($pivot, $this->m_GeneralTitle->keys())){
                return $this->m_GeneralTitle->get($pivot);
            }
        }
        return "";
    }
    function abeLOMgetGeneralTitleKeys()
    {
        //ABULEDU_LOG_TRACE
        return $this->m_GeneralTitle->keys();
    }

    /**
      * 1.3: Langue
      * @attention cardinalité n
      * @param value langue(s) du document
      * @see Dublin Core  DC.Language ISO639
      * @see http://www.loc.gov/standards/iso639-5/id.php
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:language>fre</lom:language>
<lom:language>eng</lom:language>
      * @endcode
      */
    function abeLOMsetGeneralLanguage($lang){
        //ABULEDU_LOG_TRACE
        if(is_array($lang)){

            foreach ($lang as $value) {
                array_push($this->m_GeneralLanguage, $value);
            }
            return true;
        }else{
            if(array_key_exists($lang, $this->m_availableLanguages)){
                array_push($this->m_GeneralLanguage, $lang);
                return true;
            }else{
                if(in_array($lang, $this->m_availableLanguages)){
                    array_push($this->m_GeneralLanguage, array_search($lang, $this->m_availableLanguages));
                    return true;
                }
            }
        }
        return false;
    }

    function abeLOMgetGeneralLanguage(){
        //ABULEDU_LOG_TRACE
        if(count($this->m_GeneralLanguage) == 0){
            array_push($this->m_GeneralLanguage, "fre");
        }
        return $this->m_GeneralLanguage;
    }


    function abeLOMaddGeneralLanguage($lang){
        //ABULEDU_LOG_TRACE
        if(array_key_exists($lang, $this->m_availableLanguages)){
            if(!in_array($lang, $this->m_GeneralLanguage)){
                array_push($this->m_GeneralLanguage, $lang);
                return true;
            }
        }else{
            if(in_array($lang, $this->m_availableLanguages)){
                $pivot = array_search($lang, $this->m_availableLanguages);
                if(!in_array($pivot, $this->m_GeneralLanguage)){
                    array_push($this->m_GeneralLanguage, $pivot);
                    return true;
                }
            }
        }
        return false;
    }

    /**
      * 1.4: Description
      * @attention cardinalité n
      * @param lang langue de la description
      * @param value la description dans la langue en question
      * @see Dublin Core  DC.Description
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:description>
  <lom:string language="fre">Une description super intéressante de la ressource dans la langue 'fre'</lom:string>
</lom:description>
<lom:description>
  <lom:string language="eng">The same but in english</lom:string>
</lom:description>
      * @endcode
      */
    function abeLOMsetGeneralDescription($lang, $value){
        //ABULEDU_LOG_TRACE
        if(empty($value)){
            return false;
        }

        if(array_key_exists($lang, $this->m_availableLanguages)){
           $this->m_GeneralDescription->set($lang, $value);
           return true;
        }else{
            if(array_search($lang, $this->m_availableLanguages)){
                $this->m_GeneralDescription->set(array_search($lang, $this->m_availableLanguages),$value);
                return true;
            }else{
                return false;
            }
        }
    }

    function abeLOMaddGeneralDescription($lang, $value){
        //ABULEDU_LOG_TRACE
        if(array_search($lang, $this->abeLOMgetAvailableLanguages())){
            $lang = array_search($lang, $this->abeLOMgetAvailableLanguages());
        }
        //ABULEDU_LOG_DEBUG

        if(empty($lang)){
            return false;
        }else {
            if(array_key_exists($lang, $this->abeLOMgetAvailableLanguages())){
                $this->m_GeneralDescription->add($lang, $value);
                return true;
            }
        }
        return false;
    }

    function abeLOMclearGeneralDescription(){
        //ABULEDU_LOG_TRACE
        $this->m_GeneralDescription->clear();
    }

    function abeLOMgetGeneralDescription($lang){
        if(array_search($lang, $this->m_availableLanguages)){
            $lang = array_search($lang, $this->m_availableLanguages);
        }else if(!array_key_exists($lang, $this->m_availableLanguages)){
            $tmp = array();
            return $tmp;
        }
        //ABULEDU_LOG_DEBUG

        return $this->m_GeneralDescription->getList($lang);
    }

    function abeLOMgetGeneralDescriptionKeys(){
        //ABULEDU_LOG_TRACE
        return $this->m_GeneralDescription->keys();
    }

    /**
      * 1.5: Mots clés libres
      * @attention cardinalité n
      * @param lang langue du mot clé
      * @param value le mot clé
      * @see Dublin Core  DC.Subject
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:keyword>
  <lom:string language="fre">bateau</lom:string>
</lom:keyword>
<lom:keyword>
  <lom:string language="fre">remorqueur</lom:string>
</lom:keyword>
<lom:keyword>
  <lom:string language="eng">tug</lom:string>
</lom:keyword>
      * @endcode
      */
    function abeLOMsetGeneralKeyword($lang, $value){
        if(in_array($lang, $this->abeLOMgetAvailableLanguages())){
            $lang = array_search($lang, $this->abeLOMgetAvailableLanguages());
        }
        //ABULEDU_LOG_DEBUG
        if(empty($lang) || empty($value)){
            return false;
        }else{
            if(array_key_exists($lang, $this->abeLOMgetAvailableLanguages())){
                $this->m_GeneralKeyword->set($lang, $value);
                return true;
            }
        }
        return false;
    }


    function abeLOMaddGeneralKeyword($lang, $value){
        //ABULEDU_LOG_TRACE
         if(in_array($lang, $this->abeLOMgetAvailableLanguages())){
            $lang = array_search($lang, $this->abeLOMgetAvailableLanguages());
        }
        //ABULEDU_LOG_DEBUG
        if(empty($lang) || empty($value)){
            return false;
        }else{
            if(array_key_exists($lang, $this->abeLOMgetAvailableLanguages())){
                $this->m_GeneralKeyword->add($lang, $value);
                return true;
            }
        }    
        return false;        
    }

    function abeLOMgetGeneralKeywords($lang){
        //ABULEDU_LOG_TRACE(spe)
        if(in_array($lang, $this->abeLOMgetAvailableLanguages())){
            $lang = array_search($lang, $this->abeLOMgetAvailableLanguages());
        }

        $result;
        if(array_key_exists($lang, $this->m_availableLanguages)){
            $result = $this->m_GeneralKeyword->getList($lang);
        }

        return $result;
    }

    function abeLOMclearGeneralKeywords(){
        //ABULEDU_LOG_TRACE
        $this->m_GeneralKeyword->clear();
    }

    function abeLOMgetGeneralKeywordKeys(){
        //ABULEDU_LOG_TRACE(spe)
        return $this->m_GeneralKeyword->keys();
    }

    /**
      * 1.6: Couverture au sens large: géographique, époque (Lieu, Région, Époque)
      * @attention cardinalité n
      * @param lang langue de la couverture
      * @param value la couverture
      * @see Dublin Core  DC.Coverage
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:coverage>
 <lom:string language="fre">Empire musulman</lom:string>
</lom:coverage>
<lom:coverage>
 <lom:string language="fre">VIIIe-XVe siècle</lom:string>
</lom:coverage>
      * @endcode
      */
    function abeLOMsetGeneralCoverage($lang, $value){
        //ABULEDU_LOG_TRACE
        if(in_array($lang, $this->abeLOMgetAvailableLanguages())){
            $lang = array_search($lang, $this->abeLOMgetAvailableLanguages());
        }
        if(empty($lang) || empty($value)){
            return false;
        }else{
            if(array_key_exists($lang, $this->abeLOMgetAvailableLanguages())){
                $this->m_GeneralCoverage->set($lang, $value);
                return true;
            }
        }
        return false;
    }

    function abeLOMaddGeneralCoverage($lang, $value){
        //ABULEDU_LOG_TRACE(spe)
        if(in_array($lang, $this->abeLOMgetAvailableLanguages())){
            $lang = array_search($lang, $this->abeLOMgetAvailableLanguages());
        }   
        if(empty($lang) || empty($value)){
            return false;
        }else{
            if(array_key_exists($lang, $this->abeLOMgetAvailableLanguages())){
                $this->m_GeneralCoverage->add($lang, $value);
                //TO-DO : Supprimer les liges vides
                return true;
            }
        }    
        return false;        
    }

    function abeLOMgetGeneralCoverage($lang){
        //ABULEDU_LOG_TRACE
        if(in_array($lang, $this->abeLOMgetAvailableLanguages())){
            $lang = array_search($lang, $this->abeLOMgetAvailableLanguages());
        }

        if(in_array($lang, $this->m_GeneralCoverage->keys())){
            return $this->m_GeneralCoverage->getList($lang);
        }else{
            return "";
        }
    }

    function abeLOMgetGeneralCoverageKeys(){
        //ABULEDU_LOG_TRACE
        return $this->m_GeneralCoverage->keys();
    }

    /**
      * 1.7: Structure
      * @attention cardinalité 1
      * @attention utiliser le vocabulaire adapté : atomic, collection, hierarchical, linear, networked
      * @param value la structure (dans un vocabulaire)
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:structure>
  <lom:source>LOMv1.0</lom:source>
  <lom:value>collection</lom:value>
</lom:structure>
      * @endcode
      */
    function abeLOMsetGeneralStructure($value)
    {
        //ABULEDU_LOG_TRACE(spe)
        if(array_key_exists($value, $this->m_GeneralStructureVocabLOM)){
           
            $value = $this->m_GeneralStructureVocabLOM[$value];
        }
        if(in_array($value, $this->m_GeneralStructureVocabLOM)){
            $this->m_GeneralStructure="LOMv1.0;".$value;
            return true;
        }else{
            return false;
        }
    }

    function abeLOMgetGeneralStructure()
    {
        //ABULEDU_LOG_TRACE
        return $this->m_GeneralStructure;
    }

    function abeLOMgetGeneralStructureVocabLOM(){
        //ABULEDU_LOG_TRACE
        return $this->m_GeneralStructureVocabLOM;
    }


    /**
      * 1.8: Niveau d'agrégation
      * @attention cardinalité 1
      * @attention utiliser le vocabulaire adapté : 1,2,3,4  Attention si la ressource a été définie comme atomique alors son "aggregationLevel" doit être de 1
      * @param value de l'agrégation (dans un vocabulaire)
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:aggregationLevel>
  <lom:source>LOMv1.0</lom:source>
  <lom:value>2</lom:value>
</lom:aggregationLevel>
      * @endcode
      */
    function abeLOMsetGeneralAggregationLevel($value)
    {
        //ABULEDU_LOG_TRACE
        if(in_array($value, $this->m_GeneralAggregationLevelVocabLOM)){
            $this->m_GeneralAggregationLevel="LOMv1.0;".$value;
            return true;
        }else{
            return false;
        }
    }

    function abeLOMgetGeneralAggregationLevel()
    {
        //ABULEDU_LOG_TRACE
        return $this->m_GeneralAggregationLevel;
    }

    function abeLOMgetGeneralAggregationLevelVocabLOM(){
        //ABULEDU_LOG_TRACE
        return $this->m_GeneralAggregationLevelVocabLOM;
    }

    // ------------------- branche 2 : Cycle de vie
    /**
      * 2.1: Version
      * @attention cardinalité 1
      * @param lang la langue
      * @param value la valeur
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:version>
  <lom:string language="fre">1.4.2</lom:string>
</lom:version>
      * @endcode
      */
    function abeLOMsetLifeCycleVersion($lang, $value){
        //ABULEDU_LOG_TRACE(lange,value)
        if(empty($lang) || empty($value)){
            return false;
        }
        if(in_array($lang, $this->m_availableLanguages)){
            $lang = array_search($lang, $this->m_availableLanguages);
        }
        if(array_key_exists($lang, $this->m_availableLanguages)){
            $this->m_LifeCycleVersion->set($lang, $value);
            return true;
        }
        return false;
    }

    function abeLOMgetLifeCycleVersion($lang){

        //ABULEDU_LOG_TRACE
        if(in_array($lang, $this->m_availableLanguages)){
            $lang = array_search($lang, $this->m_availableLanguages);
        }
        

        if(in_array($lang, $this->m_LifeCycleVersion->keys())){
            return $this->m_LifeCycleVersion->get($lang);

        }else{
            return "";
        }
    }

    function abeLOMgetLifeCycleVersionKeys(){
        //ABULEDU_LOG_TRACE
        return $this->m_LifeCycleVersion->keys();
    }

    /**
      * 2.2: État
      * @attention cardinalité 1
      * @attention utiliser le vocabulaire adapté : draft, final, revised, unvailable
      * @param value valeur de l'état (dans le vocabulaire)
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:status>
  <lom:source>LOMv1.0</lom:source>
  <lom:value>final</lom:value>
</lom:status>
      * @endcode
      */
    function abeLOMsetLifeCycleStatus($value){
        //ABULEDU_LOG_TRACE
        if(in_array($value, $this->m_LifeCycleStatusVocabLOM)){
            $value = array_search($value, $this->m_LifeCycleStatusVocabLOM);
        }

        if(array_key_exists($value, $this->m_LifeCycleStatusVocabLOM)){
            $this->m_LifeCycleStatus="LOMv1.0;".$value;
            return true;
        }else{
            return false;
        }
    }

    function abeLOMgetLifeCycleStatus(){
        //ABULEDU_LOG_TRACE
        return $this->m_LifeCycleStatus;
    }

    function abeLOMgetLifeCycleStatusVocabLOM(){
        //ABULEDU_LOG_TRACE
        return $this->m_LifeCycleStatusVocabLOM;
    }

    /**
      * 2.3: Contribution (cardinalité n)
      * 2.3.1: Rôle (contribution)
      * @attention cardinalité 1
      * @attention utiliser le vocabulaire adapté : author, publisher, unknown, initiator, terminator, validator, editor, graphical designer, technical implementer, content provider, technical validator, educational validator, script writer, instructional designer, subject matter expert
      * @param value valeur de l'état (dans le vocabulaire)
      * @see Dublin Core  DC.Creator/DC.Publisher/DC.Contributor
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:contribute>
  <lom:role>
    <lom:source>LOMv1.0</lom:source>
    <lom:value>publisher</lom:value>
  </lom:role>
  <lom:entity>BEGIN:VCARD
VERSION:3.0
N:NOM;LePrénom;;;
FN:LePrénom NOM
TEL:01 02 03 04 05
LOGO:http://www.abuledu.org
EMAIL;TYPE=INTERNET:adresse@abuledu.org
NOTE;TYPE=SIREN:000 000 000
ORG:AbulÉdu
URL:
UID:login@abuledu.org
END:VCARD</lom:entity>
  <lom:date>
  <lom:dateTime>2012-04-09</lom:dateTime>
  </lom:date>
</lom:contribute>
      * @endcode
      */
    function abeLOMgetLifeCycleContributionRoleVocabLOM(){
        //ABULEDU_LOG_TRACE
        return $this->m_LifeCycleContributionRoleVocabLOM;
    }

    /**
      * Ajoute une nouvelle contribution
      */
    function abeLOMaddLifeCycleContributionRole($role, $entity, $ladate){
        //ABULEDU_LOG_TRACE(spe)
        //empecher qu'on ajoute un meme triplet

        if(!array_key_exists($role, $this->m_LifeCycleContributionRoleVocabLOM)){
            if(in_array($role, $this->m_LifeCycleContributionRoleVocabLOM)){
                $role = array_search($role, $this->m_LifeCycleContributionRoleVocabLOM);
            }else{
                //ABULEDU_LOG_DEBUG
                return false;
            }
        }

        return $this->p_abeLOMaddLifeCycleContributionRole($role, $entity, $ladate);

    }

    function p_abeLOMaddLifeCycleContributionRole($role, $entity, $ladate){
        //ABULEDU_LOG_TRACE(spe)
        /* Si l'entité est "vide" on dégage tout de suite */
        if($entity->get("FN") == ""){
            //ABULEDU_LOG_DEBUG
            return false;
        }

        //ABULEDU_LOG_DEBUG
        /* On ajoute la vcard a la liste pour pouvoir retourner la liste des vcards plus tard ... */
        if(!in_array($entity, $this->m_allVcards)){
            array_push($this->m_allVcards, $entity);
        }

        $retour = false;

        /* Si par malheur la date n'est pas bonne on passe a aujourd'hui */
        if(!$ladate instanceof DateTime){
            $ladate = new DateTime();
        }

        $recherche = $role.":".$ladate->format("Y-m-d");
        /* On ajoute une entity a la liste qui peut déjà exister, pour le savoir, on cherche la cle "recherche" */
        /* On cherche une clé qui est composée de role:date */
        if(array_key_exists($recherche, $this->m_LifeCycleContributions)){
            $listeVCardTemp = $this->m_LifeCycleContributions[$recherche];
            if(!in_array($entity, $listeVCardTemp)){
                array_push($listeVCardTemp, $entity);
                unset($this->m_LifeCycleContributions[$recherche]);
                $this->m_LifeCycleContributions[$recherche] = $listeVCardTemp;
                $retour = true;
            }
        }else{
            /* La clé n'existe pas, on ajoute tout */
            $listeVCardTemp = array();
            array_push($listeVCardTemp, $entity);
            $this->m_LifeCycleContributions[$recherche] = $listeVCardTemp;
            $retour = true;
        }

        //ABULEDU_LOG_DEBUG
            
        return $retour;

    }

    /**
      * Supprime une contribution
      */
    function abeLOMremoveLifeCycleContributionRole($role, $entity, $ladate){
        //ABULEDU_LOG_TRACE
        $retour = false;
        $recherche = $role.":".$ladate->format("Y-m-d");
        if(in_array($recherche, $this->m_LifeCycleContributions)){
            //on cherche l'entité dans cette structure
            $listeVCardTemp = $this->m_LifeCycleContributions[$recherche];
            if(in_array($entity, $listeVCardTemp)){
                while(in_array($entity, $listeVCardTemp)){
                    unset($listeVCardTemp[array_search($entity, $listeVCardTemp)]);
                }
                unset($this->m_LifeCycleContributions[$recherche]);
                $this->m_LifeCycleContributions[$recherche] = array();
                array_push($this->m_LifeCycleContributions[$recherche], $listeVCardTemp);
                $retour = true;
            }

        }
        //ABULEDU_LOG_DEBUG

        return $retour;
    }

    /**
      * Retourne la date la plus récente des contributions
      * @param role le role en question (attention doit être dans la liste du vocabulaire accepté)
      */
    function abeLOMgetLifeCycleContributionLastDate($role){
        //ABULEDU_LOG_TRACE(spe)
        $nomRole = array_search($role, $this->abeLOMgetLifeCycleContributionRoleVocabLOM());
        $dateA;
        $dateB = new DateTime("0-1-1");
        $dateOk =  false;

        if(count($this->m_LifeCycleContributions) != 0){
            foreach ($this->m_LifeCycleContributions as $key => $value) {
               
                if(strstr($key, $nomRole.":")){
                    $ladate = str_replace($nomRole.":", "", $key);
                    $dateA = DateTime::createFromFormat("Y-m-d", $ladate);

                    if($dateA > $dateB){
                        $dateOk = true;
                        $dateB = $dateA;
                    }
                }
            }
        }

        if(!$dateOk){
            $dateB = new DateTime();
        }

        return $dateB;
    }

    /**
      * Retourne une liste de vcard correspondantes au rôle ... role pour la date  ... date
      * @param role le role en question (attention doit être dans la liste du vocabulaire accepté)
      * @param date la date en question
      */
    function abeLOMgetLifeCycleContributionEntities($role, $date){
        //ABULEDU_LOG_TRACE
        $liste = array(); //Liste de Vcard

        if(!empty($role)){

            $recherche = $role.":".$date->format("Y-m-d");

            if(array_key_exists($recherche, $this->m_LifeCycleContributions)){

                array_push($liste, $this->m_LifeCycleContributions[$recherche]);
            }
        }
        if(count($liste) == 0){
            $empty = File_IMC::build('vCard');
            $empty->setFormattedName("Non renseigné");
            array_push($liste, $empty);
        }

        return $liste;
    }
    /**
      * Retourne les dates des contributions
      * @param role le role en question (attention doit être dans la liste du vocabulaire accepté)
      */
    function abeLOMgetLifeCycleContributionDates($role){
        //ABULEDU_LOG_TRACE(spe)
        //Retourne la liste des dates pour un role donné
        //la c'est pas optimal du fait de la structure que j'utilise
        //je suis obligé de parcourir toute la structure ...
        $nomRole = array_search($role, $this->abeLOMgetLifeCycleContributionRoleVocabLOM());

        //Peut-être qu'on a passé une clé ?
        if(empty($nomRole)){
            if(array_key_exists($role, $this->abeLOMgetLifeCycleContributionRoleVocabLOM())){
                $nomRole = $role;
            }


        }


        $dates = array();
        foreach ($this->m_LifeCycleContributions as $key => $value) {
            if(strstr($key, $nomRole)){
                $ladate = str_replace($nomRole.":", "", $key);
                array_push($dates, DateTime::createFromFormat("Y-m-d", $ladate));
            }
        }
        return $dates;
    }

    /**
      * Retourne la liste des rôles utilisés
      * @note liste sans doublon
      */
    function abeLOMgetLifeCycleContributionRoles(){
        //ABULEDU_LOG_TRACE
        //Tableau de String 
        $retour = array();
        if(count($this->m_LifeCycleContributions)){
            $liste = array_keys($this->m_LifeCycleContributions);

            foreach($liste as $element){
                $tmp = explode(":", $element);
                if(!in_array($tmp[0], $retour)){
                    array_push($retour, $tmp[0]);
                }
            }
        }
        return $retour;
    }


    // ------------------- branche 3: Meta MetaData
    /**
      * Branche 3: A FAIRE
      * @todo la branche 3 n'est pas du tout prioritaire et donc probablement pas implémentée, toute contribution est la bienvenue, merci d'avance
      */
    //3

    //3.1.1
    function abeLOMsetMetaMetadataIdentifierCatalog($value)
    {
        //ABULEDU_LOG_TRACE
      $this->m_MetaMetadataIdentifierCatalog=$value;
      return true;
    }
    function abeLOMgetMetaMetadataIdentifierCatalog()
    {
        //ABULEDU_LOG_TRACE

      return $this->m_MetaMetadataIdentifierCatalog;
    }

    //3.1.2
    function abeLOMsetMetaMetadataIdentifierEntry($value)
    {
        //ABULEDU_LOG_TRACE
      $this->m_MetaMetadataIdentifierEntry=$value;
      return true;
    }
    function abeLOMgetMetaMetadataIdentifierEntry()
    {
        //ABULEDU_LOG_TRACE
        return $this->m_MetaMetadataIdentifierEntry;
    }

    //3.2 Gestion Contribution (cardinalité n)
    function abeLOMaddMetaMetadataContribute($role, $entity, $date){
        $contribution = array();

        if(in_array($role, $this->m_LifeCycleContributionRoleVocabLOM)){
            $role = array_search($role, $this->m_LifeCycleContributionRoleVocabLOM);
        }

        if(!array_key_exists($role, $this->m_LifeCycleContributionRoleVocabLOM)){
            return false;
        }

        if($entity->get("FN") == ""){
            return false;
        }

        if(!$date instanceof DateTime){
            $date = new DateTime();
        }

        $contribution["role"] = $role;
        $contribution["entity"] = $entity;
        $contribution["date"] = $date;


        array_push($this->m_MetaMetadataContribute, $contribution);
        return true;

    }

    function abeLOMgetMetaMetadataContribute(){
        return $this->m_MetaMetadataContribute;
    }

    //3.2.1
    function abeLOMsetMetaMetadataContributeRole($value)
    {
        //ABULEDU_LOG_TRACE
      $this->m_MetaMetadataContributeRole=$value;
    }
    function abeLOMgetMetaMetadataContributeRole()
    {
        //ABULEDU_LOG_TRACE
      return $this->m_MetaMetadataContributeRole;
    }

    //3.2.2
    function abeLOMsetMetaMetadataContributeEntity($value)
    {
        //ABULEDU_LOG_TRACE
      $this->m_MetaMetadataContributeEntity=$value;
    }
    function abeLOMgetMetaMetadataContributeEntity()
    {
        //ABULEDU_LOG_TRACE
      return $this->m_MetaMetadataContributeEntity;
    }

    //3.2.3
    function abeLOMsetMetaMetadataContributeDate($value)
    {
        //ABULEDU_LOG_TRACE
      $this->m_MetaMetadataContributeDate=$value;
    }
    function abeLOMgetMetaMetadataContributeDate()
    {
        //ABULEDU_LOG_TRACE
      return $this->m_MetaMetadataContributeDate;
    }

    //3.3
    function abeLOMaddMetaMetadataMetadataSchema($value){
        array_push($this->m_MetaMetadataMetadataSchema, $value);
        return true;
    }

    function abeLOMgetMetaMetadataMetadataSchema()
    {
        //ABULEDU_LOG_TRACE
      return $this->m_MetaMetadataMetadataSchema;
    }

    //3.4
    function abeLOMsetMetaMetadataLanguage($value)
    {
        //ABULEDU_LOG_TRACE
        /** @todo vérifier s'il est important que ça soit une langue de type "fre" */
      $this->m_MetaMetadataLanguage=$value;
    }
    function abeLOMgetMetaMetadataLanguage()
    {
        //ABULEDU_LOG_TRACE

      return $this->m_MetaMetadataLanguage;
    }

    // ------------------- branche 4 : Technique
    /**
      * 4.1: Format, le type mime du/des document(s) numériques ou autre pour documents physiques
      * @attention cardinalité n
      * @param value le type en question
      * @see Dublin Core DC.Format
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:format>application/pdf</lom:format>
<lom:format>application/abuledu</lom:format>
      * @endcode
      */
    function abeLOMaddTechnicalFormat($value){
        //ABULEDU_LOG_TRACE
        if(empty($value)){
            return false;
        }else{
            if(!in_array($value, $this->m_TechnicalFormat)){
                array_push($this->m_TechnicalFormat, $value);
                return true;
            }
        }
        return false;
    }

    function abeLOMgetTechnicalFormat(){
        //ABULEDU_LOG_TRACE
        return $this->m_TechnicalFormat;
    }

    function abeLOMclearTechnicalFormat(){
        //ABULEDU_LOG_TRACE
        $this->m_TechnicalFormat = array();
    }

    /**
      * 4.2: Taille, la taille de la ressources en octets
      * @attention cardinalité 1
      * @param value la taille en octets
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:size>0</lom:size>
      * @endcode
      */
    function abeLOMsetTechnicalSize($value)
    {
        //ABULEDU_LOG_TRACE
        $this->m_TechnicalSize=$value;
        return true;
    }
    function abeLOMgetTechnicalSize()
    {
        //ABULEDU_LOG_TRACE
        return $this->m_TechnicalSize;
    }

    /**
      * 4.3: Localisation, chaine de caractère utilisée pour accèder à la ressource: url, uri, ou une indication pour se procurer le document
      * @attention cardinalité n
      * @param value la localisation
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:location>http://raconte-moi.abuledu.org/w/tag/djataka</lom:location>
<lom:location>http://libre.pedagosite.net/voir_doc.php?DOC=274</lom:location>
      * @endcode
      */
    function abeLOMaddTechnicalLocation($value){
        //ABULEDU_LOG_TRACE
        //trim supp
        if(empty($value)){
            return false;
        }

        if(!in_array($value, $this->m_TechnicalLocation)){
            array_push($this->m_TechnicalLocation, $value);
            return true;
        }
        return false;
    }

    function abeLOMdelTechnicalLocation($value){
        //ABULEDU_LOG_TRACE
        //trim supp
        if(empty($value)){
            return false;
        }
        if(in_array($value, $this->m_TechnicalLocation)){
            while (in_array($value, $this->m_TechnicalLocation)) {
                unset($this->m_TechnicalLocation[array_search($value, $this->m_TechnicalLocation)]);
            }
            return true;
        }

        return false;
    }

    function abeLOMgetTechnicalLocation(){
        //ABULEDU_LOG_TRACE
        $principale = $this->abeLOMgetGeneralIdentifierEntry();
        $retour = array();
        if(!in_array($principale, $this->m_TechnicalLocation)){
            if(!count($this->m_TechnicalLocation)){
                if($principale != "")   
                    array_push($retour, $principale);
            }
        }
        return array_merge($retour, $this->m_TechnicalLocation);
    }


    /**
      * Branche 4: A FINIR
      * @todo la branche 4 n'est pas totalement implémentée, toute contribution est la bienvenue, merci d'avance
      */
    //4.4.1.1
    function abeLOMsetTechnicalRequirementOrCompositeType($value)
    {
        //ABULEDU_LOG_TRACE
      $this->m_TechnicalRequirementOrCompositeType=$value;

    }
    function abeLOMgetTechnicalRequirementOrCompositeType()
    {
        //ABULEDU_LOG_TRACE
      return $this->m_TechnicalRequirementOrCompositeType;
    }

    //4.4.1.2
    function abeLOMsetTechnicalRequirementOrCompositeName($value)
    {
        //ABULEDU_LOG_TRACE
      $this->m_TechnicalRequirementOrCompositeName=$value;
    }
    function abeLOMgetTechnicalRequirementOrCompositeName()
    {
        //ABULEDU_LOG_TRACE
      return $this->m_TechnicalRequirementOrCompositeName;
    }

    //4.4.1.3
    function abeLOMsetTechnicalRequirementOrCompositeMinimumVersion($value)
    {
        //ABULEDU_LOG_TRACE
      $this->m_TechnicalRequirementOrCompositeMinimumVersion=$value;
    }
    function abeLOMgetTechnicalRequirementOrCompositeMinimumVersion()
    {
        //ABULEDU_LOG_TRACE
      return $this->m_TechnicalRequirementOrCompositeMinimumVersion;
    }

    //4.4.1.4
    function abeLOMsetTechnicalRequirementOrCompositeMaximumVersion($value)
    {
        //ABULEDU_LOG_TRACE
      $this->m_TechnicalRequirementOrCompositeMaximumVersion=$value;
    }
    function abeLOMgetTechnicalRequirementOrCompositeMaximumVersion()
    {
        //ABULEDU_LOG_TRACE
      return $this->m_TechnicalRequirementOrCompositeMaximumVersion;
    }

    //4.5
    function abeLOMsetTechnicalInstallationRemarks($lang,$value)
    {
        //ABULEDU_LOG_TRACE
        $this->m_TechnicalInstallationRemarks->set($lang,$value);
    }
    function abeLOMgetTechnicalInstallationRemarks($lang)
    {
        //ABULEDU_LOG_TRACE
      return $this->m_TechnicalInstallationRemarks->get($lang);
    }


    function abeLOMgetTechnicalInstallationRemarksKeys()
    {
        //ABULEDU_LOG_TRACE
      return $this->m_TechnicalInstallationRemarks->keys();
    }

    //4.6
    function abeLOMsetTechnicalOtherPlatformRequirements($lang,$value)
    {
        //ABULEDU_LOG_TRACE
      $this->m_TechnicalOtherPlatformRequirements->set($lang,$value);
    }
    function abeLOMgetTechnicalOtherPlatformRequirements($lang)
    {
        //ABULEDU_LOG_TRACE
      return $this->m_TechnicalOtherPlatformRequirements->get($lang);
    }
    function abeLOMgetTechnicalOtherPlatformRequirementsKeys()
    {
        //ABULEDU_LOG_TRACE
      return $this->m_TechnicalOtherPlatformRequirements->keys();
    }

    /**
      * 4.7: Durée Temps moyen nécessaire à une utilisation de la ressource pédagogique à présenter suivant le modèle ci-dessus
      * @attention cardinalité 1
      * @param value la durée PT suivi de hHmMsS, exemple PT1H15M30S
      * @note Exemple de code XML qui sera généré:
      * @code
    <lom:duration>
    <lom:duration>PT1H15M30S</lom:duration>
    </lom:duration>
      * @endcode
      */
    function abeLOMsetTechnicalDuration($value){
        //ABULEDU_LOG_TRACE
        if(!($value == "PT0H0M0S")){
            $this->m_TechnicalDuration=$value;
        }
    }

    function abeLOMgetTechnicalDuration()
    {
        //ABULEDU_LOG_TRACE
        return $this->m_TechnicalDuration;
    }

    // ------------------- branche 5 : Pédagogie
    /**
      * 5.1: Type d'interactivité: interactivityType
      * @attention attribut LOM non intégré au LOMFR
      * @attention utiliser le vocabulaire adapté :
      * @param value valeur (dans le vocabulaire)
      * @note Exemple de code XML qui sera généré:
      * @code
      * @endcode
      */
    function abeLOMsetEducationalInteractivityType($value)
    {
        //ABULEDU_LOG_TRACE
        if(in_array($value, $this->m_EducationalInteractivityTypeVocabLOM)){
            $value = array_search($value, $this->m_EducationalInteractivityTypeVocabLOM);
        }

        if(array_key_exists($value, $this->m_EducationalInteractivityTypeVocabLOM)){
            $this->m_EducationalInteractivityType="LOMv1.0;".$value;
            return true;
        }
        return false;
    }

    function abeLOMgetEducationalInteractivityTypeVocabLOM(){
        return $this->m_EducationalInteractivityTypeVocabLOM;
    }

    function abeLOMgetEducationalInteractivityType()
    {
        //ABULEDU_LOG_TRACE
      return $this->m_EducationalInteractivityType;
    }

    /**
      * 5.2: Type de ressource pédagogique dans l'idée de Nature pédagogique de la ressouce: learningResourceType
      * @attention cardinalité n
      * @attention vocabulaire LOM: exercise, simulation, questionnaire, diagram, figure, graph, index, slide, table, narrative text, exam, experiment, problem statement, self assessment, lecture
      * @attention vocabulaire LOMFR: démonstration, évaluation, animation, tutoriel, glossaire, guide, matériel de référence, méthodologie, outil, scénario pédagogique
      * @attention vocabulaire SCOLOMFR: étude de cas, jeu éducatif, manuel d'enseignement, production d'élève, annales, cyberquête, méthode de langues, témoignage pédagogique
      * @attention utiliser le vocabulaire adapté ... mais lequel ? la version française ou l'anglaise ? on aurait une préférence pour la version anglaise pour plus de "portabilité"
      * @param value valeur (dans le vocabulaire)
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:learningResourceType>
   <lom:source>LOMFRv1.0</lom:source>
   <lom:value>méthodologie</lom:value>
</lom:learningResourceType>
<lom:learningResourceType>
   <lom:source>LOMFRv1.0</lom:source>
   <lom:value>matériel de référence</lom:value>
</lom:learningResourceType>
<lom:learningResourceType>
   <lom:source>LOMv1.0</lom:source>
   <lom:value>lecture</lom:value>
</lom:learningResourceType>
      * @endcode
      */
    function abeLOMaddEducationalLearningResourceType($value){
        //ABULEDU_LOG_TRACE(spe)
        if(in_array($value, $this->m_EducationalLearningResourceTypeVocabLOM)){
            $key = array_search($value, $this->m_EducationalLearningResourceTypeVocabLOM);
            if(!in_array("LOMv1.0;".$key, $this->m_EducationalLearningResourceType)){
                array_push($this->m_EducationalLearningResourceType, "LOMv1.0;".$key);
                return true;
            }
        }else if(array_key_exists($value, $this->m_EducationalLearningResourceTypeVocabLOM)){
            if(!in_array("LOMv1.0;".$value, $this->m_EducationalLearningResourceType)){
                array_push($this->m_EducationalLearningResourceType, "LOMv1.0;".$value);
                return true;
            }
        }

        return false;
        
    }

    function abeLOMgetEducationalLearningResourceType(){
        //ABULEDU_LOG_TRACE
        return $this->m_EducationalLearningResourceType;
    }

    function abeLOMgetEducationalLearningResourceTypeVocabLOM(){
        //ABULEDU_LOG_TRACE
        return $this->m_EducationalLearningResourceTypeVocabLOM;
    }


    /**
      * 5.3: Niveau d'interactivité: interactivityLevel
      * @attention attribut LOM non intégré au LOMFR
      * @param value
      * @note Exemple de code XML qui sera généré:
      * @code
      * @endcode
      */
    function abeLOMsetEducationalInteractivityLevel($value)
    {
        //ABULEDU_LOG_TRACE
      $this->m_EducationalInteractivityLevel=$value;
    }

    function abeLOMgetEducationalInteractivityLevel()
    {
        //ABULEDU_LOG_TRACE
      return $this->m_EducationalInteractivityLevel;
    }

    /**
      * 5.4: Densité sémantique: semanticDensity
      * @attention attribut LOM non intégré au LOMFR
      * @param value
      * @note Exemple de code XML qui sera généré:
      * @code
      * @endcode
      */
    function abeLOMsetEducationalSemanticDensity($value)
    {
        //ABULEDU_LOG_TRACE
      $this->m_EducationalSemanticDensity=$value;
    }

    function abeLOMgetEducationalSemanticDensity()
    {
        //ABULEDU_LOG_TRACE
      return $this->m_EducationalSemanticDensity;
    }

    /**
      * 5.5: Rôle présumé de l'utilisateur final: intendedEndUserRole
      * @attention cardinalité n
      * @attention vocabulaire LOM: teacher, author, learner, manager
      * @param value valeur (dans le vocabulaire)
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:intendedEndUserRole>
   <lom:value>learner</lom:value>
</lom:intendedEndUserRole>
<lom:intendedEndUserRole>
   <lom:value>teacher</lom:value>
</lom:intendedEndUserRole>
      * @endcode
      */
    function abeLOMaddEducationalIntendedEndUserRole($value){
        //ABULEDU_LOG_TRACE
        if(array_key_exists($value, $this->m_EducationalIntendedEndUserRoleVocabLOM)){
            $value = $this->m_EducationalIntendedEndUserRoleVocabLOM[$value];
        }

        if(in_array($value, $this->m_EducationalIntendedEndUserRoleVocabLOM)){
            if(!in_array("LOMv1.0;".$value, $this->m_EducationalIntendedEndUserRole)){
                array_push($this->m_EducationalIntendedEndUserRole, "LOMv1.0;".$value);
            }
            return true;
        }
        return false;
    }

    function abeLOMgetEducationalIntendedEndUserRole(){
        //ABULEDU_LOG_TRACE
        return $this->m_EducationalIntendedEndUserRole;
    }

    function abeLOMgetEducationalIntendedEndUserRoleVocabLOM(){
        //ABULEDU_LOG_TRACE
        return $this->m_EducationalIntendedEndUserRoleVocabLOM;
    }


    /**
      * 5.6: Contexte et niveau du public cible: context
      * @attention cardinalité n
      * @attention vocabulaire LOM: school, higher education, training, other
      * @attention vocabulaire LOMFR: enseignement primaire, enseignement secondaire, licence, master, doctorat, formation continue, formation en entreprise
      * @param value valeur (dans le vocabulaire)
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:context>
   <lom:source>LOMFRv1.0</lom:source>
   <lom:value>enseignement secondaire</lom:value>
</lom:context>
      * @endcode
      */
    function abeLOMaddEducationalContext($value){
        //ABULEDU_LOG_TRACE
        if(in_array($value, $this->m_EducationalContextVocabLOM)){
            $key = array_search($value, $this->m_EducationalContextVocabLOM);
            if(!in_array("LOMv1.0;".$key, $this->m_EducationalContext)){
                array_push($this->m_EducationalContext, "LOMv1.0;".$key);
            }
            return true;
        }else if (array_key_exists($value, $this->m_EducationalContextVocabLOM)) {
            if(!in_array("LOMv1.0;".$value, $this->m_EducationalContext)){
                array_push($this->m_EducationalContext, "LOMv1.0;".$value);   
            }
            return true;
        }

        return false;
    }

    function abeLOMgetEducationalContext(){
        //ABULEDU_LOG_TRACE
        return $this->m_EducationalContext;
    }

    function abeLOMgetEducationalContextVocabLOM(){
        //ABULEDU_LOG_TRACE
        return $this->m_EducationalContextVocabLOM;
    }

    function abeLOMclearEducationalContext(){
        //ABULEDU_LOG_TRACE
        $this->m_EducationalContext = array();
    }

    /**
      * 5.7: Tranche d'âge typicalAgeRange
      * @attention cardinalité n
      * @param value valeur de type chaine (localisée) de caractère de type "ageMini-ageMaxi"
      * @note Exemple de code XML qui sera généré:
      * @todo Ajouter une regExp qui interdise d'enregistrer une chaine de caractère de structure différente
      * @code
<lom:typicalAgeRange>
   <lom:string language="fre">15-16</lom:string>
</lom:typicalAgeRange>
      * @endcode
      */
    function abeLOMaddEducationalTypicalAgeRange($lang, $value){
        //ABULEDU_LOG_TRACE
        if(in_array($lang, $this->abeLOMgetAvailableLanguages())){
            $lang = array_search($lang, $this->abeLOMgetAvailableLanguages());
        }
        //ABULEDU_LOG_DEBUG
        //ABULEDU_LOG_TRACE

        if(!array_key_exists($lang, $this->m_availableLanguages)){
            return false;
        }

        if(empty($lang) || empty($value) || $value=="0-0"){
            return false;
        }else{
            $this->m_EducationalTypicalAgeRange->add($lang, $value);
            return true;   
        }


        return false;
    }

    function abelomclearEducationalTypicalAgeRange(){
        return $this->m_EducationalTypicalAgeRange->clear();
    }

    function abeLOMgetEducationalTypicalAgeRange($lang){
        return $this->m_EducationalTypicalAgeRange->getList($lang);
    }

    function abeLOMgetEducationalTypicalAgeRangeKeys(){
        return $this->m_EducationalTypicalAgeRange->keys();
    }

    /**
      * 5.8: Difficulté: Difficulty
      * @attention cardinalité 1
      * @attention vocabulaire LOM: very easy, easy, medium, difficult, very difficult
      * @param value valeur (dans le vocabulaire)
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:difficulty>
   <lom:source>LOMv1.0</lom:source>
   <lom:value>medium</lom:value>
</lom:difficulty>
      * @endcode
      */
    function abeLOMsetEducationalDifficulty($value){
        if (in_array($value, $this->m_EducationalDifficultyVocabLOM)) {
            $key = array_search($value, $this->m_EducationalDifficultyVocabLOM);
            if(!strstr($this->m_EducationalDifficulty, "LOMv1.0;".$key)){
                $this->m_EducationalDifficulty = "LOMv1.0;".$key;
            }
            return true;
        }else if(array_key_exists($value, $this->m_EducationalDifficultyVocabLOM)){
            if(!strstr($this->m_EducationalDifficulty, "LOMv1.0;".$value)){
                $this->m_EducationalDifficulty = "LOMv1.0;".$value;
            }
            return true;
        }

        return false;
    }

    function abeLOMgetEducationalDifficulty(){
        //ABULEDU_LOG_TRACE
        return $this->m_EducationalDifficulty;
    }

    function abeLOMgetEducationalDifficultyLOM(){
        //ABULEDU_LOG_TRACE
        return $this->m_EducationalDifficultyVocabLOM;
    }

    /**
      * 5.9: Temps d'apprentissage moyen: typicalLearningTime
      * @attention cardinalité 1
      * @param value valeur de type Date
      * @note Exemple de code XML qui sera généré:
      * @code
<typicallearningtime>
  <datetime>PT1H20M</datetime>
</typicallearningtime>
      * @endcode
      */
    function abeLOMgetEducationalTypicalLearningTime(){
        //ABULEDU_LOG_TRACE
        return $this->m_EducationalTypicalLearningTime;
    }

    function abeLOMsetEducationalTypicalLearningTime($minute){
        //ABULEDU_LOG_TRACE
        $this->m_EducationalTypicalLearningTime->setTime(0,$minute, 0);
        return true;    
    }

    /**
      * 5.10: Description et proposition d'utilisation: description
      * @attention cardinalité n
      * @param value valeur de type chaine (localisée) de caractère
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:description>
  <lom:string language="fre">Cette ressource peut s'utiliser en vidéoprojection, en séance avec les élèves sur les postes informatiques ou à domicile.</lom:string>
</lom:description>
      * @endcode
      */
    function abeLOMaddEducationalDescription($lang, $value){
        //ABULEDU_LOG_TRACE
        if(in_array($lang, $this->m_availableLanguages)){
            $lang = array_search($lang, $this->m_availableLanguages);
        }
        //ABULEDU_LOG_DEBUG
        if(empty($lang) || empty($value)){
            return false;
        }else{
            if(array_key_exists($lang, $this->m_availableLanguages)){
                $this->m_EducationalDescription->add($lang, $value);
                return true;
            }
        }
        return false;
    }

    function abeLOMclearEducationalDescription(){
        //ABULEDU_LOG_TRACE
        $this->m_EducationalDescription->clear();
    }

    function abeLOMgetEducationalDescription($lang){
        //ABULEDU_LOG_TRACE
        return $this->m_EducationalDescription->getList($lang);
    }

    function abeLOMgetEducationalDescriptionKeys(){
        //ABULEDU_LOG_TRACE
        return $this->m_EducationalDescription->keys();
    }

    /**
      * 5.11: Langue de l'utilisateur
      * @attention cardinalité n
      * @param value valeur de type chaine (localisée) de caractères au format ISO639
      * @see http://www.loc.gov/standards/iso639-5/id.php
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:language>fre</lom:language>
<lom:language>eng</lom:language>
<lom:language>ara</lom:language>
<lom:language>spa</lom:language>
<lom:language>dut</lom:language>
<lom:language>ger</lom:language>
      * @endcode
      */
    function abeLOMaddEducationalLanguage($value){
        //ABULEDU_LOG_TRACE
        if(in_array($value, $this->m_availableLanguages)){
            $value = array_search($value, $this->m_availableLanguages);
        }

        if (array_key_exists($value, $this->m_availableLanguages)) {
            if(!in_array($value, $this->m_EducationalLanguage)){
                array_push($this->m_EducationalLanguage, $value);
                return true;
            }
        }
    }

    function abeLOMgetEducationalLanguage(){
        //ABULEDU_LOG_TRACE
        return $this->m_EducationalLanguage;
    }

    // ------------------- branche 6 : Droits & Coûts
    /**
      * 6.1: Coût, permet d'indiquer si la ressource est payante / a un coût: cost
      * @attention cardinalité 1
      * @attention vocabulaire LOM: yes, no
      * @param value valeur (dans le vocabulaire)
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:cost>
  <lom:source>LOMv1.0</lom:source>
  <lom:value>yes</lom:value>
</lom:cost>
      * @endcode
      */
    function abeLOMsetRightsCost($value){
        //ABULEDU_LOG_TRACE
        if(in_array($value, $this->abeLOMgetRightsCostVocabLOM())){
            $value = array_search($value, $this->abeLOMgetRightsCostVocabLOM());
        }
        //ABULEDU_LOG_DEBUG
        if(empty($value)){
            return false;
        }

        if(array_key_exists($value, $this->abeLOMgetRightsCostVocabLOM())){
            $this->m_RightsCost = "LOMv1.0;".$value;
            return true;
        }
        return false;
    }
    
    function abeLOMgetRightsCost(){
        //ABULEDU_LOG_TRACE
        return $this->m_RightsCost;
    }

    function abeLOMgetRightsCostVocabLOM(){
        //ABULEDU_LOG_TRACE
        return $this->m_RightsCostVocabLOM;
    }

    /**
      * 6.2: Droit d'auteur, permet d'indiquer si la ressource a un droit d'auteur ou si elle est complètement libre de droits: copyrightAndOtherRestrictions
      * @attention cardinalité 1
      * @attention vocabulaire LOM: yes, no
      * @param value valeur (dans le vocabulaire), sera généralement à yes, on a un droit d'auteur qui s'applique et c'est justement grâce à lui qu'on applique ensuite une licence libre de type cc-by-sa
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:copyrightAndOtherRestrictions>
  <lom:source>LOMv1.0</lom:source>
  <lom:value>yes</lom:value>
</lom:copyrightAndOtherRestrictions>
      * @endcode
      */
    function abeLOMsetRightsCopyrightAndOtherRestrictions($value){
        //ABULEDU_LOG_TRACE
        if(in_array($value, $this->abeLOMgetRightsCopyrightAndOtherRestrictionsVocabLOM())){
            $value = array_search($value, $this->abeLOMgetRightsCopyrightAndOtherRestrictionsVocabLOM());
        }
        //ABULEDU_LOG_DEBUG
        if(empty($value)){
            return false;
        }

        if(array_key_exists($value, $this->abeLOMgetRightsCopyrightAndOtherRestrictionsVocabLOM())){
            $this->m_RightsCopyrightAndOtherRestrictions = "LOMv1.0;".$value;
            return true;
        }

        return false;
    }

    function abeLOMgetRightsCopyrightAndOtherRestrictions(){
        //ABULEDU_LOG_TRACE
        return $this->m_RightsCopyrightAndOtherRestrictions;
    }

    function abeLOMgetRightsCopyrightAndOtherRestrictionsVocabLOM(){
        //ABULEDU_LOG_TRACE
        return $this->m_RightsCopyrightAndOtherRestrictionsVocabLOM;
    }

    /**
      * 6.3: Description, conditions légales d'utilisation de la ressource ou du document: description
      * @attention cardinalité 1
      * @param valeur de type chaine (localisée) de caractère
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:description>
  <lom:string language="fre">cc-by-sa</lom:string>
</lom:description>
      * @endcode
      */
    function abeLOMsetRightsDescription($lang, $value){
        //ABULEDU_LOG_DEBUG
        if(!empty($value) && ! empty($lang)){
            
            if(in_array($lang, $this->m_availableLanguages)){
                $lang = array_search($lang, $this->m_availableLanguages);
            }
            if(array_key_exists($lang, $this->m_availableLanguages)){
                $this->m_RightsDescription->set($lang, $value);
            }
        }

    }

    function abeLOMgetRightsDescription($lang){
        //ABULEDU_LOG_TRACE
        if(in_array($lang, $this->m_availableLanguages)){
            $lang = array_search($lang, $this->m_availableLanguages);
        }

        if(array_key_exists($lang, $this->m_availableLanguages)){
            return $this->m_RightsDescription->get($lang);
        }

        return "";
    }

    function abeLOMgetRightsDescriptionKeys(){
        //ABULEDU_LOG_TRACE
        return $this->m_RightsDescription->keys();
    }

    // ------------------- branche 7 : relations
    /**
      * 7.1: Nature de la relation : kind
      * @attention cardinalité 1
      * @attention vocabulaire LOM: ispartof, haspart, isversionof, hasversion, isformatof, hasformat, references, isreferencedby, isbasedon, isbasisfor, requires, isrequiredby
      * @attention vocabulaire LOMFR: est associée à, est la traduction de, fait l'objet d'une traduction, est la localisation de, fait l'objet d'une localisation, est prérequis de,  a pour prérequis
      * @attention vocabulaire SCOLOMFR: a pour vignette, a pour logo, a pour aperçu, est l'aperçu de
      * @param value valeur (dans le vocabulaire)
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:relation>
  <lom:kind>
    <lom:source>LOMv1.0</lom:source>
    <lom:value>ispartof</lom:value>
  </lom:kind>
  <lom:resource>
    <lom:identifier>
      <lom:catalog>nom du catalogue où la ressource est stockée</lom:catalog>
      <lom:entry> Valeur de la ressource destination avec lequel est fait le lien. Cela peut être url du document ou du chapitre ou autre moyen qui permet de retrouver le document</lom:entry>
    </lom:identifier>
    <lom:description>
      <lom:string language="fre">Description de la ressource "destination"</lom:string>
    </lom:description>
  </lom:resource>
</lom:relation>

<lom:relation>
  <lom:kind>
    <lom:source>LOMFRv1.0</lom:source>
    <lom:value>est la traduction de</lom:value>
  </lom:kind>
  <lom:resource>
    <lom:identifier>
      <lom:catalog>nom du catalogue où la ressource est stockée</lom:catalog>
      <lom:entry> Valeur de la ressource destination avec lequel est fait le lien. Cela peut être url du document ou du chapitre ou autre moyen qui permet de retrouver le document</lom:entry>
    </lom:identifier>
    <lom:description>
      <lom:string language="fre">Description de la ressource "destination"</lom:string>
    </lom:description>
  </lom:resource>
</lom:relation>
      * @endcode
      */
    function abeLOMsetRelationKind($value){
        //ABULEDU_LOG_TRACE
        if(array_key_exists($value, $this->m_RelationKindVocabLOM)){
            $value = $this->m_RelationKindVocabLOM[$value];
        }

        if(in_array($value, $this->m_RelationKindVocabLOM)){
            $this->m_RelationKind = "LOMv1.0;".$value;
            return true;
        }
        return false;
    }

    function abeLOMgetRelationKind(){
        //ABULEDU_LOG_TRACE
        return $this->m_RelationKind;
    }

    function abeLOMgetRelationKindVocabLOM(){
        //ABULEDU_LOG_TRACE
        return $this->m_RelationKindVocabLOM;
    }

    /**
      * 7.2: Ressource : resource
      * 7.2.1: Identifiant : identifier
      *
      * 7.2.1.1: catalogue
      * 7.2.1.2: Entrée : entry
      * @attention cardinalité n
      * @param value valeur chaîne de caractères
      * @see abeLOMsetRelationKind()
      * Comme on peut avoir plusieurs URI,http://bnmablablabla une QMap n'est pas adaptée,
      * choix d'avoir une qstringlist de type URI;http://jgdklgd << URI;http://glgegre
      */
    function abeLOMaddRelationResourceIdentifier($catalog, $entry){
        //ABULEDU_LOG_TRACE
        array_push($this->m_RelationResourceIdentifierEntry, $catalog.";".$entry);
    }

    function abeLOMdelRelationResourceIdentifier($catalog, $entry){
        //ABULEDU_LOG_TRACE
        $value = $catalog.";".$entry;

        //trim supp
        if(!empty($value)){
            if(in_array($value, $this->m_RelationResourceIdentifierEntry)){
                while(in_array($value, $this->m_RelationResourceIdentifierEntry)){
                    unset($this->m_RelationResourceIdentifierEntry[array_search($value, $this->m_RelationResourceIdentifierEntry)]);
                }
            }
        }
    }

    function abeLOMgetRelationResourceIdentifier(){
        //ABULEDU_LOG_TRACE
        return $this->m_RelationResourceIdentifierEntry;
    }

    /**
      * 7.2: Ressource : resource
      * 7.2.2: Description de la ressource "destination": description
      * @attention cardinalité n
      * @param value valeur de type chaine (localisée) de caractère
      * @see abeLOMsetRelationKind()
      */
    function abeLOMaddRelationResourceDescription($lang, $value){
        //ABULEDU_LOG_TRACE
        if(in_array($lang, $this->m_availableLanguages)){
            $lang = array_search($lang, $this->m_availableLanguages);
        }

        if(array_key_exists($lang, $this->m_availableLanguages)){
            $this->m_RelationResourceDescription->add($lang, $value);
        }     
    }

    function abeLOMgetRelationResourceDescription($lang){
        //ABULEDU_LOG_TRACE
        if(in_array($lang, $this->m_availableLanguages)){
            $lang = array_search($lang, $this->m_availableLanguages);
        }

        if(array_key_exists($lang, $this->m_availableLanguages)){
            return $this->m_RelationResourceDescription->get($lang);
        }
    }

    function abeLOMgetRelationResourceDescriptionKeys(){
        //ABULEDU_LOG_TRACE
        return $this->m_RelationResourceDescription->keys();
    }

    //Gestion cardinalité n des relations
    function abeLOMaddRelation($type, $catalogue, $entree, $description){
        if(array_key_exists($type, $this->m_RelationKindVocabLOM)){
            $type = $this->m_RelationKindVocabLOM[$type];
        }

        if(in_array($type, $this->m_RelationKindVocabLOM)){

            if(!array_key_exists("LOMv1.0;".$type, $this->m_Relation)){
                $this->m_Relation["LOMv1.0;".$type] = array();
                $this->m_Relation["LOMv1.0;".$type]["catalogue"] = $catalogue;
                $this->m_Relation["LOMv1.0;".$type]["entree"] = $entree;
                $this->m_Relation["LOMv1.0;".$type]["description"] = $description;

            }
            return true;
        }

        return false;        
    }

    function abeLOMgetRelation(){
        return $this->m_Relation;
    }

    // ------------------- branche 8 : Annotations
    function abeLOMaddAnnotation($entity, $ladate, $description){

        if(!$ladate instanceof DateTime){
            $ladate = new DateTime();
        }
        if($entity->get("FN") == ""){
            return false;
        }



        $annotationTab = array();

        $annotationTab["entity"] = $entity;
        $annotationTab["date"] = $ladate;
        $annotationTab["description"] = $description;
        

        array_push($this->m_Annotation, $annotationTab);

        return true;
    }

    function abeLOMgetAnnotation(){
        return $this->m_Annotation;
    }

    /**
      * 8: Annotations, cette catégorie permet de stocker des remarques libres sur la ressource: annotation. Cardinalité n
      * 8.1: Entité au format vcard: entity
      * @attention cardinalité 1
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:annotation>
  <lom:entity>[BEGIN:VCARD VERSION:3.0 N:Nom; Prenom;;; FN:Nom Prenom ORG:Nom de l'organisme du "contributeur" EMAIL;INTERNET:Nom@adresse.fr END:VCARD]</lom:entity>
  <lom:date>
    <lom:dateTime>2012-04-15</lom:dateTime>
  </lom:date>
  <lom:description>
    <lom:string language="fre">Contenu du commentaire sur l'utilisation de la ressource</lom:string>
  </lom:description>
</lom:annotation>
      * @endcode
      */
    function abeLOMsetAnnotationEntity($value){
        //ABULEDU_LOG_TRACE
        if(!in_array($value, $this->m_allVcards)){
            array_push($this->m_allVcards, $value);
        }

        $this->m_AnnotationEntity = $value;
    }

    function abeLOMgetAnnotationEntity(){
        //ABULEDU_LOG_TRACE
        return $this->m_AnnotationEntity;
    }

    /**
      * 8.2: La date : date
      * @attention cardinalité 1
      * @param value valeur de type QDate
      * @see abeLOMsetAnnotationEntity();
      */
    function abeLOMsetAnnotationDate($value){
        //ABULEDU_LOG_TRACE
        $this->m_AnnotationDate->setDate($value);
    }

    function abeLOMgetAnnotationDate(){
        //ABULEDU_LOG_TRACE
        return $this->m_AnnotationDate;
    }

    /**
      * 8.3: La description: description
      * @attention cardinalité 1
      * @param value valeur de type chaine (localisée) de caractère
      * @see abeLOMsetAnnotationEntity();
      */
    function abeLOMsetAnnotationDescription($lang, $value){
        //ABULEDU_LOG_TRACE
        if(in_array($lang, $this->m_availableLanguages)){
            $lang = array_search($lang, $this->m_availableLanguages);
        }

        if(array_key_exists($lang, $this->m_availableLanguages)){
            $this->m_AnnotationDescription->set($lang, $value);
        }     
    }

    function abeLOMgetAnnotationDescription($lang){
        //ABULEDU_LOG_TRACE
        if(in_array($lang, $this->m_availableLanguages)){
            $lang = array_search($lang, $this->m_availableLanguages);
        }

        if(array_key_exists($lang, $this->m_availableLanguages)){
            return $this->m_AnnotationDescription->get($lang);
        }
    }

    function abeLOMgetAnnotationDescriptionKeys(){
        //ABULEDU_LOG_TRACE
        return $this->m_AnnotationDescription->keys();
    }

    // ------------------- branche 9 : Classification
    /**
      * 9: Classification, ctte catégorie décrit où cette ressource se trouve dans un système de classification particulier: classification. Cardinalité n
      * 9.1: Objectif: purpose
      * @attention cardinalité 1
      * @attention vocabulaire LOM: discipline, idea, prerequisite, educational objective, accessibility restrictions, educational level, skill level, security level, competency
      * @attention vocabulaire LOMFR:
      * @attention vocabulaire SCOLOMFR: enseignement, public cible détaillé, label, type de diffusion
      * @attention vocabulaire SCOLOMFR decembre 2012 beaucoup plus détaillé: http://www.lom-fr.fr/scolomfr/vocabulaires/consultation-des-vocabulaires.html
      * @param value valeur (dans le vocabulaire)
      * @note Exemple de code XML qui sera généré:
      * @code
<lom:classification>
  <lom:purpose>
    <lom:source>LOMv1.0</lom:source>
    <lom:value>discipline</lom:value>
  </lom:purpose>
  <lom:taxonPath>
    <lom:source>
      <lom:string language="fre">CDD 22e éd.</lom:string>
      <lom:string language="eng">DDC 22nd ed</lom:string>
    </lom:source>
    <lom:taxon>
      <lom:id>identifiant du taxon si vous utilisez la DDC: mettre l'indice</lom:id>
      <lom:entry>
        <lom:string language="fre">"Etiquette" texte du taxon</lom:string>
      </lom:entry>
    </lom:taxon>
  </lom:taxonPath>
  <lom:description>
    <lom:string language="fre">Description textuelle relative à la classification</lom:string>
  </lom:description>
  <lom:keyword>
    <lom:string language="fre">Mots clés relatifs à l'objectif de la classification</lom:string>
  </lom:keyword>
</lom:classification>
      * @endcode
      */
    function abeLOMgetClassificationPurpose($racine){
        //ABULEDU_LOG_TRACE
        if(array_key_exists($racine, $this->m_ClassificationPurpose)){
            return $this->m_ClassificationPurpose[$racine];
        }
    }

    /** Retourne le nombre d'objectifs présents dans la structure de données */
    function abeLOMgetClassificationPurposeNumber(){
        //ABULEDU_LOG_TRACE
        return count($this->m_ClassificationPurpose);
    }

    /** Retourne l'index correspondant à la value, ou -1 si la value n'est pas dans la QHash */
    function abeLOMgetClassificationPurposeIndex($value){
        //ABULEDU_LOG_TRACE
        $retour = -1;

        if(in_array($value, $this->m_ClassificationPurpose)){
            $retour = array_search($value, $this->m_ClassificationPurpose);
        }
        return $retour;
    }

    function abeLOMgetClassificationPurposeVocabLOM(){
        //ABULEDU_LOG_TRACE
        return $this->m_ClassificationPurposeVocabLOM;
    }

    /**
      * 9.2 : Chemin taxum / taxon : taxonPath. Cardinalité n
      * 9.2.1 : Source: source
      * @attention cardinalité 1
      * @param num le numero (entier) de ce Taxon
      * @param lang la langue
      * @param value valeur de type chaine (localisée) de caractère
      * @see abeLOMsetClassificationPurpose();
      */
    function abeLOMsetClassificationTaxonPathSource($racine, $lang, $value){
        //ABULEDU_LOG_TRACE

        if(in_array($lang, $this->m_availableLanguages)){
            $lang = array_search($lang, $this->m_availableLanguages);
        }

        if(array_key_exists($lang, $this->m_availableLanguages)){
            $this->m_ClassificationTaxonPathSource["racine"] = new LomLS($lang, $value);
        }
    }
    function abeLOMgetClassificationTaxonPathSource($racine, $lang){
        //ABULEDU_LOG_TRACE
         if(in_array($lang, $this->m_availableLanguages)){
            $lang = array_search($lang, $this->m_availableLanguages);
        }

        if(array_key_exists($lang, $this->m_availableLanguages)){
            if(array_key_exists($racine, $this->m_ClassificationTaxonPathSource)){
                $l = $this->m_ClassificationTaxonPathSource[$racine];
                return $l->get($lang);
            }
        }       
    }

    /** Retourne l'index correspondant au LomLS(lang,value), ou -1 s'il n'existe pas dans la QHash */
    function abeLOMgetClassificationTaxonPathIndexForSource($lang, $value){
        //ABULEDU_LOG_TRACE
        //ABULEDU_LOG_WARN("This method is not implemented. Return -1")
        return -1;
    }

    function abeLOMgetClassificationTaxonPathSourceKeys($num){
        //ABULEDU_LOG_TRACE
        if(array_key_exists($num, $this->m_ClassificationTaxonPathSource)){
            $l = $this->m_ClassificationTaxonPathSource[$num];   
            return $l->keys(); 
        }

        return array();    
    }

    function abeLOMgetClassificationTaxonPathSourceSize(){
        //ABULEDU_LOG_TRACE
        return count($this->m_ClassificationTaxonPathSource);
    }

    /* Fonctions liées à ma nouvelle structure de données (attention : je mets "purpose" au lieu d'"objectif" pour ne pas tout renommer si on part comme ça */
    function abeLOMgetClassificationPurposes(){
        //ABULEDU_LOG_TRACE
        return $this->m_classificationPurposes;
    }

    /** Efface le contenu de la structure de données. Appelée avant de faire les "add", de façon à éviter les doublons */
    function abeLOMclearClassificationPurposes(){
        //ABULEDU_LOG_TRACE
        $this->m_classificationPurposes = array();
    }

    /** Ajoute les éléments d'un nouveau bloc <classification> dans le XML */
    function abeLOMaddClassificationPurpose($purpose, $sourceTaxonPath, $idTaxon, $entryTaxon) {
        //ABULEDU_LOG_TRACE
        $aocPurpose = "";

        if(array_key_exists($purpose, $this->m_ClassificationPurposeVocabLOM)){
            $aocPurpose = "LOMv1.0;".$purpose;
        }else if(in_array($purpose, $this->m_ClassificationPurposeVocabLOM)){
            $aocPurpose = "LOMv1.0;".array_search($purpose, $this->m_ClassificationPurposeVocabLOM);
        }else if(strpos($purpose, "ScoLOMFRv1.0") !== false){
            $aocPurpose = $purpose;
        }else{
            return false;
        }

        //ABULEDU_LOG_DEBUG

        $taxonPath = array(); //QHash<QString(source),QHash(taxons)<QString(id),QString(LomLS) > >
        $taxons = array(); //QHash<QString(id), QString(LomLS) >

        if(array_key_exists($aocPurpose, $this->m_classificationPurposes)){
            $taxonPath = $this->m_classificationPurposes[$aocPurpose]; //ok

            if(array_key_exists($sourceTaxonPath->toString(), $taxonPath)){
                $taxons = $taxonPath[$sourceTaxonPath->toString()];
            }
        }

        /* Je n'ai pas besoin de vérifier si on avait déjà une entry pour cette id : si elle y était insert l'écrase, et l'ajoute sinon */
        if(!array_key_exists($idTaxon, $taxons)){
            //$taxons[$idTaxon] = array();
        }

        //Si l'id est null ?? on ajoute l'entrée en utilisant array_push. l'id sera un nombre
        if($idTaxon == "" || $idTaxon == ""){
            array_push($taxons, $entryTaxon);
        }else
            $taxons[$idTaxon] = $entryTaxon;


        /**foreach ($entryTaxon as $value){
            array_push($taxons[$idTaxon], $value->toString());
        }*/

        //array_push($taxons[$idTaxon], $entryTaxon->toString());

        //ABULEDU_LOG_DEBUG

        /* J'écrase l'ancienne value de la key sourceTaxonPath */
        $taxonPath[$sourceTaxonPath->toString()] = $taxons;

        //ABULEDU_LOG_DEBUG


        /* J'écrase l'ancienne value de la key purpose */
        $this->m_classificationPurposes[$aocPurpose] = $taxonPath;
        //ABULEDU_LOG_DEBUG

        return true;
    }

    /**
      * 9.2.2 : Taxum / Taxon: taxon. Cardinalité n
      * 9.2.2.1 : ID: id
      * @attention cardinalité 1
      * @param value valeur de type chaine de caractère
      * @see abeLOMsetClassificationPurpose();
      */
    function abeLOMaddClassificationTaxonPathTaxumID($racine, $num, $value){
        //ABULEDU_LOG_TRACE(spe)
        if(count($this->m_ClassificationTaxonPathTaxumID) == 0){
            $this->m_ClassificationTaxonPathTaxumID[$racine] = array();
            $this->m_ClassificationTaxonPathTaxumID[$racine][$num] = array();
            array_push($this->m_ClassificationTaxonPathTaxumID[$racine][$num], $value);
        }else{
            array_push($this->m_ClassificationTaxonPathTaxumID[$racine][$num], $value);
        }
    }

    /** retourne le nombre de taxons dans cette racine */
    function abeLOMgetClassificationTaxonPathNbTaxon($racine){
        //ABULEDU_LOG_TRACE
        return count($this->m_ClassificationTaxonPathTaxumID[$racine]);
    }

    function abeLOMgetClassificationTaxonPathTaxumID($racine, $num){
        //ABULEDU_LOG_TRACE
        if(count($this->m_ClassificationTaxonPathTaxumID) == 0){
            return "";
        }else{
            return $this->m_ClassificationTaxonPathTaxumID[$racine][$num];
        }
    }

    /**
      * 9.2.2.2 : Entrée: entry
      * @attention cardinalité 1
      * @param value valeur de type chaine (localisée) de caractère
      * @see abeLOMsetClassificationPurpose();
      */
    function abeLOMsetClassificationTaxonPathTaxumEntry($racine, $num, $lang, $value){
        //ABULEDU_LOG_TRACE(spe)
        if(in_array($lang, $this->m_availableLanguages)){
            $lang = array_search($lang, $this->m_availableLanguages);
        }

        if(!array_key_exists($lang, $this->m_availableLanguages)){
            return;
        }

        if(count($this->m_ClassificationTaxonPathTaxumEntry) == 0){ 
            $this->m_ClassificationTaxonPathTaxumEntry[$racine][$num] = array();
            $var = new LomLS($lang, $value);
            array_push($this->m_ClassificationTaxonPathTaxumEntry[$racine][$num], $var);
        }else{
            $this->m_ClassificationTaxonPathTaxumEntry[$racine][$num]->set($lang, $value);
        }
    }

    function abeLOMgetClassificationTaxonPathTaxumEntry($racine, $num, $lang){
        //ABULEDU_LOG_TRACE
          if(in_array($lang, $this->m_availableLanguages)){
            $lang = array_search($lang, $this->m_availableLanguages);
        }

        if(!array_key_exists($lang, $this->m_availableLanguages)){
            return;
        }
        
        return $this->m_ClassificationTaxonPathTaxumEntry[$racine][$num]->get($lang);     
    }

    function abeLOMgetClassificationTaxonPathTaxumEntryKeys($racine, $num){
        //ABULEDU_LOG_TRACE
        return $this->m_ClassificationTaxonPathTaxumEntry[$racine][$num]->keys();
    }
    
    /**
      * 9.3 : Description : description
      * @attention cardinalité 1
      * @param value valeur de type chaine (localisée) de caractère
      * @see abeLOMsetClassificationPurpose();
      */
    function abeLOMsetClassificationDescription($lang, $value){
        //ABULEDU_LOG_TRACE
        if(in_array($lang, $this->m_availableLanguages)){
            $lang = array_search($lang, $this->m_availableLanguages);
        }

        if(!array_key_exists($lang, $this->m_availableLanguages)){
            return;
        }

        $this->m_ClassificationDescription->set($lang,$value);
    }

    function abeLOMgetClassificationDescription($lang){
        //ABULEDU_LOG_TRACE

        if(in_array($lang, $this->m_availableLanguages)){
            $lang = array_search($lang, $this->m_availableLanguages);
        }

        if(array_key_exists($lang, $this->m_availableLanguages)){
            return $this->m_ClassificationDescription->get($lang); 
        }
        return "";
    }

    function abeLOMgetClassificationDescriptionKeys(){
        //ABULEDU_LOG_TRACE
        return $this->m_ClassificationDescription->keys();
    }

    /**
      * 9.4 : Mot clé: keyword: les mots clés de type "texte libre"
      * @attention cardinalité n
      * @param value valeur de type chaine (localisée) de caractère
      * @see abeLOMsetClassificationPurpose();
      */
    function abeLOMaddClassificationKeyword($lang, $value){
        //ABULEDU_LOG_TRACE
        if(in_array($lang, $this->m_availableLanguages)){
            $lang = array_search($lang, $this->m_availableLanguages);
        }
        //ABULEDU_LOG_DEBUG
        if(empty($lang) || empty($value)){
            return false;
        }else{
            if(!in_array($value, $this->m_ClassificationKeyword->getList($lang))){
                $this->m_ClassificationKeyword->add($lang,$value);
            }
            return true;
        }        

        return false;
    }
    function abeLOMsetClassificationKeyword($lang, $value){
        //ABULEDU_LOG_TRACE
        if(in_array($lang, $this->m_availableLanguages)){
            $lang = array_search($lang, $this->m_availableLanguages);
        }
        //ABULEDU_LOG_DEBUG
        if(empty($lang) || empty($value)){
            return false;
        }else{
            if(array_key_exists($lang, $this->m_availableLanguages)){
                $this->m_ClassificationKeyword->set($lang,$value);
                return true;
            }           
        }
        return false;
    }

    function abeLOMgetClassificationKeyword($lang){
        //ABULEDU_LOG_TRACE
        return $this->m_ClassificationKeyword->get($lang);   
    }

    function abeLOMgetClassificationKeywordKeys(){
        //ABULEDU_LOG_TRACE
        return $this->m_ClassificationKeyword->keys();
    }


    /**
      * Nettoyage de la structure de données
      */
    function clear(){
        //ABULEDU_LOG_TRACE

        $this->m_allVcards= array();
        $this->m_domDocument = new DomDocument();
        //$this->m_domRootElement = null;
        //$this->m_domCurrentElement = null;

        // ------------------- branche 1
        $this->m_GeneralIdentifierCatalog = "URI";
        $this->m_GeneralIdentifierEntry = "";
        $this->m_GeneralTitle = new LomLS();
        $this->m_GeneralLanguage= array();
        $this->m_GeneralDescription = new LomLS();
        $this->m_GeneralKeyword = new LomLS();
        $this->m_GeneralCoverage = new LomLS();
        $this->m_GeneralStructure = "";
        $this->m_GeneralAggregationLevel = "";

        // ------------------- branche 2
        $this->m_LifeCycleVersion = new LomLS();
        $this->m_LifeCycleStatus = "";
        $this->m_LifeCycleContributions = array();

        // ------------------- branche 3
        $this->m_MetaMetadata = array(); //inutil 
        $this->m_MetaMetadataContribute = array();
        $this->m_MetaMetadataIdentifierCatalog = "";
        $this->m_MetaMetadataIdentifierEntry = "";
        $this->m_MetaMetadataContributeRole = "";
        $this->m_MetaMetadataContributeEntity = "";
        $this->m_MetaMetadataContributeDate = "";
        $this->m_MetaMetadataMetadataSchema = array();
        $this->m_MetaMetadataLanguage = "";

        // ------------------- branche 4
        $this->m_TechnicalFormat = array();
        $this->m_TechnicalSize = "";
        $this->m_TechnicalLocation = array();
        $this->m_TechnicalRequirementOrCompositeType = "";
        $this->m_TechnicalRequirementOrCompositeName = "";
        $this->m_TechnicalRequirementOrCompositeMinimumVersion = "";
        $this->m_TechnicalRequirementOrCompositeMaximumVersion = "";
        $this->m_TechnicalInstallationRemarks = new LomLS();
        $this->m_TechnicalOtherPlatformRequirements = new LomLS();
        $this->m_TechnicalDuration = "";

        // ------------------- branche 5
        $this->m_EducationalInteractivityType = "";
        $this->m_EducationalLearningResourceType = array();
        $this->m_EducationalInteractivityLevel = "";
        $this->m_EducationalSemanticDensity = "";
        $this->m_EducationalIntendedEndUserRole = array();
        $this->m_EducationalContext = array();
        $this->m_EducationalTypicalAgeRange = new LomLS();
        $this->m_EducationalDifficulty = "";
        //m_EducationalTypicalLearningTime;
        $this->m_EducationalDescription = new LomLS();
        $this->m_EducationalLanguage = array();

        // ------------------- branche 6
        $this->m_RightsCost = "";
        $this->m_RightsCopyrightAndOtherRestrictions = "";
        $this->m_RightsDescription = new LomLS();

        // ------------------- branche 7
        $this->m_Relation = array();
        $this->m_RelationKind = "";
        $this->m_RelationResourceIdentifierEntry = array();
        $this->m_RelationResourceDescription = new LomLS();
        $this->m_Relation = array();

        // ------------------- branche 8
        //    m_AnnotationEntity
        //    m_AnnotationDate
        $this->m_Annotation = array();
        $this->m_AnnotationDescription = new LomLS();

        // ------------------- branche 9
        $this->m_ClassificationPurpose = array();
        $this->m_ClassificationTaxonPathSource = array();
        $this->m_ClassificationTaxonPathTaxumID = array();
        $this->m_ClassificationTaxonPathTaxumEntry = array();

        $this->m_ClassificationDescription = new LomLS();
        $this->m_ClassificationKeyword = new LomLS();

        return true;

    }

    /**
      * Exporte l'entête du fichier XML
      *
      * @note Exemple de code XML qui sera généré:
      * @code
<?xml version="1.0" encoding="UTF-8"?>
<lom:lom xmlns:lom="http://ltsc.ieee.org/xsd/LOM">
      * @endcode
      */
    function p_abeLOMExportAsXML_header(){
        //ABULEDU_LOG_TRACE
        $this->m_domDocument = new DOMDocument('1.0', 'UTF-8');
        
        $this->m_domDocument->preserveWhiteSpace = false;
        $this->m_domDocument->formatOutput = true;
        $officeDocContent = $this->m_domDocument->createElementNS("http://ltsc.ieee.org/xsd/LOM", "lom:lom");
        
        $officeDocContent->setAttribute("xsi:schemaLocation","http://ltsc.ieee.org/xsd/LOM http://ltsc.ieee.org/xsd/lomv1.0/lom.xsd");
        $officeDocContent->setAttribute("xmlns:xsi","http://www.w3.org/2001/XMLSchema-instance");
        $this->m_domDocument->appendChild($officeDocContent);     
        $this->m_domRootElement = $officeDocContent;

        return $this->m_domDocument->saveXML();
    }

    /**
      * Exporte la section 1 Général
      *
      * @note Exemple de code XML qui sera généré:
      * @code
   <lom:general>
      <lom:identifier>
         <lom:catalog>URI</lom:catalog>
         <lom:entry>http://mediatheque.abuledu.org/URI/5343xdfg32</lom:entry>
      </lom:identifier>
      <lom:title>
         <lom:string language="fre">Le titre de la ressource</lom:string>
      </lom:title>
      <lom:language>fre</lom:language>
      <lom:description>
         <lom:string language="fre">Résumé de la ressource.</lom:string>
      </lom:description>
      <lom:keyword>
         <lom:string language="fre">19e-20e siècle</lom:string>
      </lom:keyword>
      <lom:keyword>
         <lom:string language="fre">lutte politique, résistance : politique</lom:string>
      </lom:keyword>
      <lom:structure>
         <lom:source>LOMv1.0</lom:source>
         <lom:value>networked</lom:value>
      </lom:structure>
      <lom:aggregationLevel>
         <lom:source>LOMv1.0</lom:source>
         <lom:value>2</lom:value>
      </lom:aggregationLevel>
   </lom:general>
      * @endcode
      */
    function p_abeLOMExportAsXML_section1(){
        //ABULEDU_LOG_TRACE
        $e ;
        $e_fils;
        $e_fils2;
        $t;

        //============= General 1
        $e = $this->m_domDocument->createElement($this->m_xmlPrefixLom."general");
        $this->m_domRootElement->appendChild($e);
        {
            // 1.1
            $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."identifier");
            $e->appendChild($e_fils);
            {
                $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."catalog");
                $e_fils->appendChild($e_fils2);
                $t = $this->m_domDocument->createTextNode($this->m_GeneralIdentifierCatalog);
                $e_fils2->appendChild($t);

                $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."entry");
                $e_fils->appendChild($e_fils2);
                $t = $this->m_domDocument->createTextNode($this->m_GeneralIdentifierEntry);
                $e_fils2->appendChild($t);
            }

            //1.2
            if(count($this->m_GeneralTitle)){
                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."title");
                $e->appendChild($e_fils);
                {
                    foreach ($this->abeLOMgetGeneralTitleKeys() as $lang) {
                        $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."string");
                        $e_fils2->setAttribute("language",$lang);
                        $e_fils->appendChild($e_fils2);
                        //$e->appendChild($e_fils);
                        $t = $this->m_domDocument->createTextNode($this->m_GeneralTitle->get($lang));
                        $e_fils2->appendChild($t);                
                    }
                }
            }


            //1.3
            if(count($this->m_GeneralLanguage)){
                foreach ($this->abeLOMgetGeneralLanguage() as $lang) {
                   
                    $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."language");
                    $e->appendChild($e_fils);
                    $t = $this->m_domDocument->createTextNode($lang);
                    $e_fils->appendChild($t);                   
                }
            }

            //1.4
            if(count($this->m_GeneralKeyword)){
                //1ere boucle pour les langues
                foreach ($this->m_GeneralDescription->keys() as $lang) {
                    //Seconde boucle pour les mots cles
                    foreach ($this->m_GeneralDescription->getList($lang) as $description) {
                        $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."description");
                        $e->appendChild($e_fils);
                        $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."string");
                        $e_fils2->setAttribute("language",$lang);
                        $e_fils->appendChild($e_fils2);

                        $t = $this->m_domDocument->createTextNode($description);

                        $e_fils2->appendChild($t);                        
                    }
                }
            }
            
            //1.5
            if(count($this->m_GeneralKeyword)){
                //1ere boucle pour les langues
                foreach ($this->m_GeneralKeyword->keys() as $lang) {
                    //Seconde boucle pour les mots cles
                    foreach ($this->m_GeneralKeyword->getList($lang) as $value) {
                        $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."keyword");
                        $e->appendChild($e_fils);
                        $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."string");
                        $e_fils2->setAttribute("language",$lang);
                        $e_fils->appendChild($e_fils2);

                        $t = $this->m_domDocument->createTextNode($value);

                        $e_fils2->appendChild($t);                       
                    }
                }
            }

            //1.6
            if(count($this->m_GeneralCoverage->keys())){
                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."coverage");
                $e->appendChild($e_fils);
                {
                    foreach ($this->abeLOMgetGeneralCoverageKeys() as $lang) {
                        $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."string");
                        $e_fils2->setAttribute("language",$lang);
                        $e_fils->appendChild($e_fils2);
                        $t = $this->m_domDocument->createTextNode($this->m_GeneralCoverage->get($lang));
                        $e_fils2->appendChild($t);                       
                    }
                }           
            }

            //1.7
            if(!empty($this->m_GeneralStructure)){
                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."structure");
                $e->appendChild($e_fils);   
                {
                    $contenu = explode(";", $this->m_GeneralStructure);
                    $cle = array_shift($contenu);
                    $valeur = implode(";", $contenu);

                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."source");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($cle);
                    $e_fils2->appendChild($t);

                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."value");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($valeur);
                    $e_fils2->appendChild($t);                    
                }             
            }

            //1.8
            if(!empty($this->m_GeneralAggregationLevel)){
                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."aggregationLevel");
                $e->appendChild($e_fils);
                {
                    $contenu = explode(";", $this->m_GeneralAggregationLevel);
                    $cle = array_shift($contenu);
                    $valeur = implode(";", $contenu);

                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."source");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($cle);
                    $e_fils2->appendChild($t);

                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."value");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($valeur);
                    $e_fils2->appendChild($t);             
                }          
            }
        }

        $this->m_domCurrentElement = $e;

        return $this->m_domDocument->saveXML();
    }

    /**
      * Exporte la section 2 Cycle de vie
      *
      * @note Exemple de code XML qui sera généré:
      * @code
    <lom:lifeCycle>
        <lom:version>
            <lom:string language="fre">Un intitulé libre de version</lom:string>
        </lom:version>
        <lom:status>
            <lom:source>LOMv1.0</lom:source>
            <lom:value>final</lom:value>
        </lom:status>
        <lom:contribute>
            <lom:role>
                <lom:source>LOMv1.0</lom:source>
                <lom:value>author</lom:value>
            </lom:role>
            <lom:entity>BEGIN:VCARD
VERSION:3.0
N:Nom; Prenom;;;
FN:Nom Prenom
ORG:Nom de l'organisme du "contributeur";;
EMAIL;INTERNET:adresse.email@domaine.fr
END:VCARD
            </lom:entity>
            <lom:date>
                <lom:dateTime>2012-04-17</lom:dateTime>
                <lom:description>
                    <lom:string language="fre">un petit commentaire</lom:string>
                </lom:description>
            </lom:date>
        </lom:contribute>
    </lom:lifeCycle>
      * @endcode
      */
    function p_abeLOMExportAsXML_section2(){
        //ABULEDU_LOG_TRACE
        $vide = true;
        $e ;
        $e_fils;
        $e_fils2;
        $e_fils3;
        $t;

        $e = $this->m_domDocument->createElement($this->m_xmlPrefixLom."lifeCyle");
        {
            if(count($this->abeLOMgetLifeCycleVersionKeys())){
                $vide = false;

                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."version");
                $e->appendChild($e_fils);
                {
                    foreach ($this->abeLOMgetLifeCycleVersionKeys() as $lang) {
                       
                        $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."string");
                        $e_fils2->setAttribute("language",$lang);
                        $e_fils->appendChild($e_fils2);
                        $t = $this->m_domDocument->createTextNode($this->m_LifeCycleVersion->get($lang));
                        $e_fils2->appendChild($t);                       
                    }
                }

                //2.2
                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."status");
                $e->appendChild($e_fils);
                {
                    $contenu = explode(";", $this->m_LifeCycleStatus);
                    $cle = array_shift($contenu);
                    $valeur = implode(";", $contenu);  
 
                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."source");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($cle);
                    $e_fils2->appendChild($t);

                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."value");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($valeur);
                    $e_fils2->appendChild($t);                                      
                }              
            }

            //Les contributions, il faut juste faire une boucle de la structure et générer le code
            //TODO a faire apres avoir implementé la classe VCard..
            foreach ($this->m_LifeCycleContributions as $key => $value) {
                $vide = false;
                $roleEtDate = $key;
                $entitees = $value;
                $contenu = explode(":", $roleEtDate);
                $role = $contenu[0];
                $ladate = $contenu[1]; 

                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."contribute");
                $e->appendChild($e_fils);
                //Le Role
                {
                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."role");
                    $e_fils->appendChild($e_fils2);
                    {
                        $e_fils3 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."source");
                        $e_fils2->appendChild($e_fils3);
                        $t = $this->m_domDocument->createTextNode("LOMv1.0");
                        $e_fils3->appendChild($t);

                        $e_fils3 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."value");
                        $e_fils2->appendChild($e_fils3);
                        $t = $this->m_domDocument->createTextNode($role);
                        $e_fils3->appendChild($t);
                    }
                }

                //Les entités, 2° itérateur / boucle
                foreach ($entitees as $entity) {
                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."entity");
                    $e_fils->appendChild($e_fils2);
                    {
                        //vCard v = entitees.at($entity_i);
                        $t = $this->m_domDocument->createTextNode($entity->fetch());
                        $e_fils2->appendChild($t);
                    }                    
                } 

                //La date
                $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."date");
                $e_fils->appendChild($e_fils2);
                {
                    $e_fils3 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."dateTime");
                    $e_fils2->appendChild($e_fils3);
                    $t = $this->m_domDocument->createTextNode($ladate);
                    $e_fils3->appendChild($t);
                }
            }
        }

        if(!$vide){
            $this->m_domRootElement->appendChild($e);
             $this->m_domCurrentElement = $e;
        }
        return $this->m_domDocument->saveXML();
    }

    /**
      * Exporte la section 3 Méta Métadonnées
      *
      * @note Exemple de code XML qui sera généré:
      * @code
      * @endcode
      */
    function p_abeLOMExportAsXML_section3(){
        //ABULEDU_LOG_TRACE(spe)
        $vide = false;
        $e;
        $e_fils;
        $e_fils2;
        $t;

        //============= General 1
        $e = $this->m_domDocument->createElement($this->m_xmlPrefixLom."metaMetadata");
        $this->m_domRootElement->appendChild($e);
        {
            // 1.1
            $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."identifier");
            $e->appendChild($e_fils);
            {
                $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."catalog");
                $e_fils->appendChild($e_fils2);
                $t = $this->m_domDocument->createTextNode($this->abeLOMgetMetaMetadataIdentifierCatalog());
                $e_fils2->appendChild($t);

                $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."entry");
                $e_fils->appendChild($e_fils2);
                $t = $this->m_domDocument->createTextNode($this->abeLOMgetMetaMetadataIdentifierEntry());
                $e_fils2->appendChild($t);
            }
            foreach ($this->m_MetaMetadataContribute as $contrib) {

                $vide = false;
                $entitees = $contrib["entity"];
                $role = $contrib["role"];
                $ladate = $contrib["date"]->format("Y-m-d"); 
                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."contribute");
                $e->appendChild($e_fils);
                //Le Role
                {
                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."role");
                    $e_fils->appendChild($e_fils2);
                    {

                        $e_fils3 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."source");
                        $e_fils2->appendChild($e_fils3);
                        $t = $this->m_domDocument->createTextNode("LOMv1.0");
                        $e_fils3->appendChild($t);

                        $e_fils3 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."value");
                        $e_fils2->appendChild($e_fils3);
                        $t = $this->m_domDocument->createTextNode($role);
                        $e_fils3->appendChild($t);
                    }
                }

                //Les entités, 2° itérateur / boucle
                //foreach ($entitees as $entity) {
                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."entity");
                    $e_fils->appendChild($e_fils2);
                    {
                        //vCard v = entitees.at($entity_i);
                        $t = $this->m_domDocument->createTextNode($entitees->fetch());
                        $e_fils2->appendChild($t);
                    }                    
                //} 

                //La date
                $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."date");
                $e_fils->appendChild($e_fils2);
                {
                    $e_fils3 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."dateTime");
                    $e_fils2->appendChild($e_fils3);
                    $t = $this->m_domDocument->createTextNode($ladate);
                    $e_fils3->appendChild($t);
                }
            }
            foreach ($this->abeLOMgetMetaMetadataMetadataSchema() as $schema) {

                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."metadataSchema");
                $e->appendChild($e_fils);
                $t = $this->m_domDocument->createTextNode($schema);
                $e_fils->appendChild($t);    
            }

            if($this->abeLOMgetMetaMetadataLanguage() != ""){
                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."language");
                $e->appendChild($e_fils);
                $t = $this->m_domDocument->createTextNode($this->abeLOMgetMetaMetadataLanguage());
                $e_fils->appendChild($t);  
            }
        } 

        if(!$vide){
            $this->m_domRootElement->appendChild($e);
             $this->m_domCurrentElement = $e;
        }       
        return $this->m_domDocument->saveXML();
    }

    /**
      * Exporte la section 4 Technique
      *
      * @note Exemple de code XML qui sera généré:
      * @code
   <lom:technical>
      <lom:format>text/html</lom:format>
      <lom:size>125</lom:size>
      <lom:location>http://www.ryxeo.com/</lom:location>
      <lom:format>image/jpeg</lom:format>
      <lom:format>video/msvideo</lom:format>
   </lom:technical>
      * @endcode
      * @todo
      */
    function p_abeLOMExportAsXML_section4(){
        //ABULEDU_LOG_TRACE
        $vide = true;
        $e ;
        $e_fils;
        $e_fils2;
        $t;

        //============= Technique - technical

        //4.1 Format     
        $e = $this->m_domDocument->createElement($this->m_xmlPrefixLom ."technical");
        {
            if(count($this->abeLOMgetTechnicalFormat())){
                $vide = false;
                foreach ($this->abeLOMgetTechnicalFormat() as  $valeur) {

                    $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."format");
                    $e->appendChild($e_fils);
                    $t = $this->m_domDocument->createTextNode($valeur);
                    $e_fils->appendChild($t);
                }
            }
        }

        //4.2 Taille
        {
            if(!empty($this->m_TechnicalSize)){
                $vide = false;
                $valeur = $this->abeLOMgetTechnicalSize();
                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."size");
                $e->appendChild($e_fils);
                $t = $this->m_domDocument->createTextNode($valeur);
                $e_fils->appendChild($t);                
            }
        }

        //4.3 Location
        {
            if(count($this->abeLOMgetTechnicalLocation())) {
                $vide=false;
               
                foreach ($this->abeLOMgetTechnicalLocation() as $valeur){
                    $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."location");
                    $e->appendChild($e_fils);
                    $t = $this->m_domDocument->createTextNode($valeur);
                    $e_fils->appendChild($t);
                }
            }
        }       

        if(!$vide){
            $this->m_domRootElement->appendChild($e);
             $this->m_domCurrentElement = $e;
        }

        return $this->m_domDocument->saveXML();
    }

    /**
      * Exporte la section 5 Pédagogie
      *
      * @note Exemple de code XML qui sera généré:
      * @code
 <lom:educational>
  <lom:learningResourceType>
   <lom:source>LOMv1.0</lom:source>
   <lom:value>simulation</lom:value>
  </lom:learningResourceType>
  <lom:learningResourceType>
   <lom:source>LOMFRv1.0</lom:source>
   <lom:value>animation</lom:value>
  </lom:learningResourceType>
  <lom:learningResourceType>
   <lom:source>LOMv1.0</lom:source>
   <lom:value>lecture</lom:value>
  </lom:learningResourceType>
  <lom:intendedEndUserRole>
   <lom:source>LOMv1.0</lom:source>
   <lom:value>learner</lom:value>
  </lom:intendedEndUserRole>
  <lom:intendedEndUserRole>
   <lom:source>LOMv1.0</lom:source>
   <lom:value>teacher</lom:value>
  </lom:intendedEndUserRole>
  <lom:context>
   <lom:source>LOMFRv1.0</lom:source>
   <lom:value>enseignement secondaire</lom:value>
  </lom:context>
  <lom:context>
   <lom:source>LOMv1.0</lom:source>
   <lom:value>other</lom:value>
  </lom:context>
  <lom:typicalAgeRange>
   <lom:string language="eng">kids</lom:string>
  </lom:typicalAgeRange>
  <lom:typicalAgeRange>
   <lom:string language="fre">13-18 ans</lom:string>
  </lom:typicalAgeRange>
  <lom:typicalAgeRange>
   <lom:string language="fre">ado</lom:string>
  </lom:typicalAgeRange>
  <lom:typicalAgeRange>
   <lom:string language="fre">jeunes adultes</lom:string>
  </lom:typicalAgeRange>
  <lom:difficulty>
   <lom:source>LOMv1.0</lom:source>
   <lom:value>easy</lom:value>
  </lom:difficulty>
  <lom:typicalLearningTime>
   <lom:source>LOMv1.0</lom:source>
   <lom:value>PT00H15M00S</lom:value>
  </lom:typicalLearningTime>
  <lom:description>
   <lom:string language="fre">Description de l'utilisation de cette ressource ...</lom:string>
  </lom:description>
  <lom:language>
   <lom:source>LOMv1.0</lom:source>
   <lom:value>fre</lom:value>
  </lom:language>
  <lomfr:activity>
   <lomfr:source>LOMFRv1.0</lomfr:source>
   <lomfr:value>apprendre</lomfr:value>
  </lomfr:activity>
  <lomfr:activity>
   <lomfr:source>LOMFRv1.0</lomfr:source>
   <lomfr:value>observer</lomfr:value>
  </lomfr:activity>
  <lomfr:activity>
   <lomfr:source>LOMFRv1.0</lomfr:source>
   <lomfr:value>s'informer</lomfr:value>
  </lomfr:activity>
  <scolomfr:place>
   <scolomfr:source>ScoLOMFRv1.0</scolomfr:source>
   <scolomfr:value>en salle de classe</scolomfr:value>
  </scolomfr:place>
  <scolomfr:place>
   <scolomfr:source>ScoLOMFRv1.0</scolomfr:source>
   <scolomfr:value>en CDI</scolomfr:value>
  </scolomfr:place>
  <scolomfr:educationalMethod>
   <scolomfr:source>ScoLOMFRv1.0</scolomfr:source>
   <scolomfr:value>aide didactique individualisée</scolomfr:value>
  </scolomfr:educationalMethod>
  <scolomfr:educationalMethod>
   <scolomfr:source>ScoLOMFRv1.0</scolomfr:source>
   <scolomfr:value>en autonomie</scolomfr:value>
  </scolomfr:educationalMethod>
  <scolomfr:educationalMethod>
   <scolomfr:source>ScoLOMFRv1.0</scolomfr:source>
   <scolomfr:value>en groupe de compétences</scolomfr:value>
  </scolomfr:educationalMethod>
  <scolomfr:educationalMethod>
   <scolomfr:source>ScoLOMFRv1.0</scolomfr:source>
   <scolomfr:value>en classe entière</scolomfr:value>
  </scolomfr:educationalMethod>
  <scolomfr:tool>
   <scolomfr:source>ScoLOMFRv1.0</scolomfr:source>
   <scolomfr:value>vidéoprojecteur</scolomfr:value>
  </scolomfr:tool>
  <scolomfr:tool>
   <scolomfr:source>ScoLOMFRv1.0</scolomfr:source>
   <scolomfr:value>logiciels RIP</scolomfr:value>
  </scolomfr:tool>
 </lom:educational>
      * @endcode
      */
    function p_abeLOMExportAsXML_section5(){
        //ABULEDU_LOG_TRACE

        $vide = true;
        $e ;
        $e_fils;
        $e_fils2;
        $t;

        //============= classification
        $e = $this->m_domDocument->createElement($this->m_xmlPrefixLom."educational");
        {
            //5.1
            //trim supp
            if($this->abeLOMgetEducationalInteractivityType() != ""){
                //ABULEDU_LOG_DEBUG
                $vide = false;
                //util? $this->abeLOMgetEducationalInteractivityType() return un string simple (sans ';' )
                $contenu = explode(";", $this->abeLOMgetEducationalInteractivityType());
                $cle = array_shift($contenu);
                $valeur = implode(";", $contenu); 

                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."interactivityType");
                $e->appendChild($e_fils);
                {

                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."source");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($cle);
                    $e_fils2->appendChild($t);

                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."value");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($valeur);
                    $e_fils2->appendChild($t);
                }              
            }

            //5.2
            if(count($this->abeLOMgetEducationalLearningResourceType())){
                //ABULEDU_LOG_DEBUG
                $vide = false;
                foreach ($this->abeLOMgetEducationalLearningResourceType() as $result) {
                    $contenu = explode(";", $result);
                    $cle = array_shift($contenu);
                    $valeur = implode(";", $contenu);

                    $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."learningResourceType");
                    $e->appendChild($e_fils);
                    {
                        $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."source");
                        $e_fils->appendChild($e_fils2);
                        $t = $this->m_domDocument->createTextNode($cle);
                        $e_fils2->appendChild($t);

                        $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."value");
                        $e_fils->appendChild($e_fils2);
                        $t = $this->m_domDocument->createTextNode($valeur);
                        $e_fils2->appendChild($t);
                    }
                }
            }

            //5.3
            if($this->abeLOMgetEducationalInteractivityLevel() != ""){
                //ABULEDU_LOG_DEBUG
                $vide = false;
                $valeur = $this->abeLOMgetEducationalInteractivityLevel();
                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."interactivityLevel");
                $e->appendChild($e_fils);
                {
                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."source");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode("LOMv1.0");
                    $e_fils2->appendChild($t);

                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."value");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($valeur);
                    $e_fils2->appendChild($t);
                }
            }

            //5.4
            if($this->abeLOMgetEducationalSemanticDensity() != ""){
                //ABULEDU_LOG_DEBUG
                $vide = false;
                $valeur = $this->abeLOMgetEducationalSemanticDensity();
                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."semanticDensity");
                $e->appendChild($e_fils);
                {
                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."source");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode("LOMv1.0");
                    $e_fils2->appendChild($t);

                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."value");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($valeur);
                    $e_fils2->appendChild($t);
                }
            }


            //5.5
            if(count($this->abeLOMgetEducationalIntendedEndUserRole())){
                //ABULEDU_LOG_DEBUG
                $vide = false;
                foreach ($this->abeLOMgetEducationalIntendedEndUserRole() as $result) {
                    $contenu = explode(";", $result);
                    $cle = array_shift($contenu);
                    $valeur = implode(";", $contenu);


                    $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."intendedEndUserRole");
                    $e->appendChild($e_fils);
                    {
                        $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."source");
                        $e_fils->appendChild($e_fils2);
                        $t = $this->m_domDocument->createTextNode($cle);
                        $e_fils2->appendChild($t);

                        $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."value");
                        $e_fils->appendChild($e_fils2);
                        $t = $this->m_domDocument->createTextNode($valeur);
                        $e_fils2->appendChild($t);
                    }                    
                }
            }

            //5.6
            if(count($this->abeLOMgetEducationalContext())){
                //ABULEDU_LOG_DEBUG
                $vide = false;
                foreach ($this->abeLOMgetEducationalContext() as $result) {
                    $contenu = explode(";", $result);
                    $cle = array_shift($contenu);
                    $valeur = implode(";", $contenu);

                    $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."context");
                    $e->appendChild($e_fils);
                    {
                        $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."source");
                        $e_fils->appendChild($e_fils2);
                        $t = $this->m_domDocument->createTextNode($cle);
                        $e_fils2->appendChild($t);

                        $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."value");
                        $e_fils->appendChild($e_fils2);
                        $t = $this->m_domDocument->createTextNode($valeur);
                        $e_fils2->appendChild($t);
                    }                    
                }
            }


            //5.7
            if(count($this->abeLOMgetEducationalTypicalAgeRangeKeys())){
                //ABULEDU_LOG_DEBUG
                $vide = false;
                foreach ($this->abeLOMgetEducationalTypicalAgeRangeKeys() as $lang) {
                    foreach ($this->abeLOMgetEducationalTypicalAgeRange($lang) as $value) {
                        $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."typicalAgeRange");
                        $e->appendChild($e_fils);

                        $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."string");
                        $e_fils2->setAttribute("language",$lang);
                        $e_fils->appendChild($e_fils2);
                        $t = $this->m_domDocument->createTextNode($value);
                        $e_fils2->appendChild($t);
                    }
                }
            }


            //5.8
            if($this->abeLOMgetEducationalDifficulty() != ""){
                //ABULEDU_LOG_DEBUG
                $vide = false;
                $contenu = explode(";", $this->abeLOMgetEducationalDifficulty());
                $cle = array_shift($contenu);
                $valeur = implode(";", $contenu);

                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."difficulty");
                $e->appendChild($e_fils);
                {
                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."source");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($cle);
                    $e_fils2->appendChild($t);

                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."value");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($valeur);
                    $e_fils2->appendChild($t);
                }
            }

            //5.9
            $time = new DateTime('0-0-0');
            $time->setTime(0,0,0);
            if($this->abeLOMgetEducationalTypicalLearningTime() != $time){
                //ABULEDU_LOG_DEBUG
                $vide = false;
                $valeur = $this->abeLOMgetEducationalTypicalLearningTime()->format("\P\TH\Hi\Ms\S");
                if(!empty($valeur)){
                    $vide = false;
                    $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."typicalLearningTime");
                    $e->appendChild($e_fils);
                    {
                        $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."source");
                        $e_fils->appendChild($e_fils2);
                        $t = $this->m_domDocument->createTextNode("LOMv1.0");
                        $e_fils2->appendChild($t);

                        $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."value");
                        $e_fils->appendChild($e_fils2);
                        $t = $this->m_domDocument->createTextNode($valeur);
                        $e_fils2->appendChild($t);
                    }                   
                }
            }

            //5.10
            if(count($this->abeLOMgetEducationalDescriptionKeys())){
                //ABULEDU_LOG_DEBUG
                $vide = false;
                foreach ($this->abeLOMgetEducationalDescriptionKeys() as $lang) {

                    foreach ($this->abeLOMgetEducationalDescription($lang) as $value) {
                        $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."description");
                        $e->appendChild($e_fils);

                        $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."string");
                        $e_fils2->setAttribute("language",$lang);
                        $e_fils->appendChild($e_fils2);
                        $t = $this->m_domDocument->createTextNode($value);
                        $e_fils2->appendChild($t);                    
                    }
                }
            }

            //5.11
            if(count($this->abeLOMgetEducationalLanguage())){
                //ABULEDU_LOG_DEBUG
                $vide = false;
                foreach ($this->abeLOMgetEducationalLanguage() as $valeur) {
                    $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."language");
                    $t = $this->m_domDocument->createTextNode($valeur);
                    $e_fils->appendChild($t);
                    $e->appendChild($e_fils);                   
                }
            }

        }

        if(!$vide){
            $this->m_domRootElement->appendChild($e);
             $this->m_domCurrentElement = $e;
        }
        //ABULEDU_LOG_DEBUG
        return $this->m_domDocument->saveXML();       
    }

    /**
      * Exporte la section 6 Droits
      *
      * @note Exemple de code XML qui sera généré:
      * @code
 <lom:rights>
  <lom:cost>
   <lom:source>LOMv1.0</lom:source>
   <lom:value>yes</lom:value>
  </lom:cost>
  <lom:copyrightAndOtherRestrictions>
   <lom:source>LOMv1.0</lom:source>
   <lom:value>yes</lom:value>
  </lom:copyrightAndOtherRestrictions>
  <lom:description>
   <lom:string language="fre">cc-by-sa</lom:string>
  </lom:description>
 </lom:rights>
      * @endcode
      */
    function p_abeLOMExportAsXML_section6(){
        //ABULEDU_LOG_TRACE
        $vide = true;
        $e;
        $e_fils;
        $e_fils2;
        $t;

        //============= rights
        $e = $this->m_domDocument->createElement($this->m_xmlPrefixLom ."rights");
        {
            //6.1
            if($this->abeLOMgetRightsCost() != ""){
                $vide = false;
                $contenu = explode(";", $this->abeLOMgetRightsCost());
                $cle = array_shift($contenu);
                $valeur = implode(";", $contenu);    
               
                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."cost");
                $e->appendChild($e_fils);
                {
                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."source");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($cle);
                    $e_fils2->appendChild($t);

                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."value");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($valeur);
                    $e_fils2->appendChild($t);
                }                           
            }

            //6.2
            if($this->abeLOMgetRightsCopyrightAndOtherRestrictions() != ""){
                $vide = false;
                $contenu = explode(";", $this->abeLOMgetRightsCopyrightAndOtherRestrictions());
                $cle = array_shift($contenu);
                $valeur = implode(";", $contenu);

                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."copyrightAndOtherRestrictions");
                $e->appendChild($e_fils);
                {
                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."source");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($cle);
                    $e_fils2->appendChild($t);

                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."value");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($valeur);
                    $e_fils2->appendChild($t);
                }
            }

            //6.3
            if(count($this->abeLOMgetRightsDescriptionKeys())){
                $vide = false;

                foreach ($this->abeLOMgetRightsDescriptionKeys() as $lang) {
                    $value = $this->abeLOMgetRightsDescription($lang);
                    $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."description");
                    $e->appendChild($e_fils);

                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."string");
                    $e_fils2->setAttribute("language",$lang);
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($value);
                    $e_fils2->appendChild($t);
                }
            }
        }

        if(!$vide){
            $this->m_domRootElement->appendChild($e);
             $this->m_domCurrentElement = $e;
        }

        return $this->m_domDocument->saveXML();
    }

    /**
      * Exporte la section 7 Relation
      *
      * @note Exemple de code XML qui sera généré:
      * @code
 <lom:relation>
  <lom:kind>
   <lom:source>LOMv1.0</lom:source>
   <lom:value>references</lom:value>
  </lom:kind>
  <lom:resource>
   <lom:identifier>
    <lom:catalog>URI</lom:catalog>
    <lom:entry>http://www.ryxeo.com/</lom:entry>
   </lom:identifier>
   <lom:description>
    <lom:string language="fre">Présentation de la ressource.
Là c'est un test avec un lien vers le site de la société.</lom:string>
   </lom:description>
  </lom:resource>
 </lom:relation>
      * @endcode
      */
    function p_abeLOMExportAsXML_section7(){
        //ABULEDU_LOG_TRACE
        $vide = true;
        $e;
        $e_fils;
        $e_fils2;
        $e_fils3;
        $t;

        foreach ($this->m_Relation as $key => $value) {
            $vide = false;
            $e = $this->m_domDocument->createElement($this->m_xmlPrefixLom ."relation");
            {
                //7.1 Kind
                $contenu = explode(";", $key);
                $cle = array_shift($contenu);
                $valeur = implode(";", $contenu);

                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."kind");
                $e->appendChild($e_fils);
                {
                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."source");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($cle);
                    $e_fils2->appendChild($t);

                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."value");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($valeur);
                    $e_fils2->appendChild($t);
                }

                //7.2 Ressources                 
                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."resource");
                $e->appendChild($e_fils);
                {
                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."identifier");
                    $e_fils->appendChild($e_fils2);

                    $catalog = $value["catalogue"];
                    $entry = $value["entree"];

                    $e_fils3 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."catalog");
                    $t = $this->m_domDocument->createTextNode($catalog);
                    $e_fils3->appendChild($t);
                    $e_fils2->appendChild($e_fils3);

                    $e_fils3 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."entry");
                    $t = $this->m_domDocument->createTextNode($entry);
                    $e_fils3->appendChild($t);
                    $e_fils2->appendChild($e_fils3);                    
                    

                    //boucle pour les differentes langues
                    $vide = false;
                    foreach ($value["description"]->keys() as $lang) {
                     
                        foreach($value["description"]->getList($lang) as $desc){
                           // $value = $liste->get($lang);
                            $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."description");
                            $e_fils->appendChild($e_fils2);

                            $e_fils3 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."string");
                            $e_fils3->setAttribute("language",$lang);
                            $e_fils2->appendChild($e_fils3);
                            $t = $this->m_domDocument->createTextNode($desc);
                            $e_fils3->appendChild($t);                       
                        }
                    }
                }  
            }  
            if(!$vide){
                $this->m_domRootElement->appendChild($e);
                $this->m_domCurrentElement = $e;
            }              
        }

        return $this->m_domDocument->saveXML();
    }

    /**
      * Exporte la section 8 Annotation
      *
      * @note Exemple de code XML qui sera généré:
      * @code
      * @endcode
      */
    function p_abeLOMExportAsXML_section8(){
        //ABULEDU_LOG_TRACE
        //ABULEDU_LOG_TRACE
        $vide = true;
        $e;
        $e_fils;
        $e_fils2;
        $e_fils3;
        $t;

        foreach ($this->m_Annotation as $annotation) {

            $vide = false;
            $e = $this->m_domDocument->createElement($this->m_xmlPrefixLom ."annotation");
            {
                //Entity
                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."entity");
                $e->appendChild($e_fils);
                {
                    
                    $t = $this->m_domDocument->createTextNode($annotation["entity"]->fetch());
                    $e_fils->appendChild($t);
                }

                //date
                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."date");
                $e->appendChild($e_fils);
                {
                    //DateTime
                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."dateTime");
                    $e_fils->appendChild($e_fils2);
                    {
                        $t = $this->m_domDocument->createTextNode($annotation["date"]->format("Y-m-d"));
                        $e_fils2->appendChild($t);
                    }
                }

                //Description
                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."description");
                $e->appendChild($e_fils);
                {
                    foreach ($annotation["description"]->keys() as $langAnnotation) {
                            $description = $annotation["description"]->get($langAnnotation);
                            $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."string");
                            $e_fils->appendChild($e_fils2);
                            $e_fils2->setAttribute("language",$langAnnotation);

                            $t = $this->m_domDocument->createTextNode($description);
                            $e_fils2->appendChild($t);
                        
                    }
                }                          
            }

            if(!$vide){
                $this->m_domRootElement->appendChild($e);
                $this->m_domCurrentElement = $e;
            } 
        }

        return $this->m_domDocument->saveXML();       
    }

    /**
      * Exporte la section 9 Classification
      *
      * @note Exemple de code XML qui sera généré:
      * @code
 <lom:classification>
  <lom:purpose>
   <lom:source>LOMv1.0</lom:source>
   <lom:value>educational level</lom:value>
  </lom:purpose>
  <lom:taxonPath>
   <lom:source>
    <lom:string language="fre">nomenclature niveaux Sceren V 2.1</lom:string>
   </lom:source>
   <lom:taxon>
    <lom:id></lom:id>
    <lom:entry>
     <lom:string language="fre">voies générale et technologique collège cycle d'orientation 3e</lom:string>
    </lom:entry>
   </lom:taxon>
   <lom:taxon>
    <lom:id></lom:id>
    <lom:entry>
     <lom:string language="fre">voies générale et technologique lycée</lom:string>
    </lom:entry>
   </lom:taxon>
   <lom:taxon>
    <lom:id></lom:id>
    <lom:entry>
     <lom:string language="fre">voie professionnelle lycée CAP CAP Santé-Social</lom:string>
    </lom:entry>
   </lom:taxon>
  </lom:taxonPath>
 </lom:classification>
      * @endcode
      */
    function p_abeLOMExportAsXML_section9(){
        //ABULEDU_LOG_TRACE
        $vide = true;
        $e;
        $e_fils;
        $e_fils2;
        $e_fils3;
        $e_fils4;
        $t;

        //ABULEDU_LOG_DEBUG
        foreach ($this->m_classificationPurposes as $purpose => $tabTaxonPath) {
            
            //9.1
            $e = $this->m_domDocument->createElement($this->m_xmlPrefixLom."classification");
            {
            
                $vide = false;
                $contenu = explode(";", $purpose);
                $cle = array_shift($contenu);
                $valeur = implode(";", $contenu);

                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."purpose");
                $e->appendChild($e_fils);
                {
                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."source");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($cle);
                    $e_fils2->appendChild($t);

                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."value");
                    $e_fils->appendChild($e_fils2);
                    $t = $this->m_domDocument->createTextNode($valeur);
                    $e_fils2->appendChild($t);
                }
                
            }

            //9.2
            //ABULEDU_LOG_DEBUG
            foreach ($tabTaxonPath as $sourceTaxonPath => $tabTaxon) {

                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."taxonPath");
                $e->appendChild($e_fils);
                {
                    $source = new LomLS();
                    $source->fromString($sourceTaxonPath);
                    //ABULEDU_LOG_DEBUG

                    //$valueSource = $source->get($langSource);
                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."source");
                    $e_fils->appendChild($e_fils2);
                    foreach ($source->keys() as $lang) {
                        foreach ($source->getList($lang) as $valeurSource) {
                            $e_fils3 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."string");
                            $e_fils3->setAttribute("language",$lang);
                            $e_fils2->appendChild($e_fils3);

                            $t = $this->m_domDocument->createTextNode($valeurSource); //on ne prend qu'une seule source
                            $e_fils3->appendChild($t);
                        }
                    }
                }

                foreach ($tabTaxon as $idTaxon => $value) {
                    $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."taxon");
                    $e_fils->appendChild($e_fils2);
                    {
                        $e_fils3 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."id");
                        $e_fils2->appendChild($e_fils3);
                        $t = $this->m_domDocument->createTextNode($idTaxon);
                        $e_fils3->appendChild($t);

                        
                        $e_fils3 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."entry");
                        $e_fils2->appendChild($e_fils3);

                        if($value != null){
                            foreach ($value->keys() as $langEntry) {

                                foreach ($value->getList($langEntry) as $entry) {
                                    $e_fils4 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."string");
                                    $e_fils4->setAttribute("language",$langEntry);
                                    $e_fils3->appendChild($e_fils4);
                                    $t = $this->m_domDocument->createTextNode($entry);
                                    $e_fils4->appendChild($t);
                                }
                            }
                        }
                    }
                }
            }
            
            if(!$vide) {
                $this->m_domRootElement->appendChild($e);
                $this->m_domCurrentElement = $e;
            }     
        }

        //9.3
        if(count($this->abeLOMgetClassificationDescriptionKeys())){
            foreach ($this->abeLOMgetClassificationDescriptionKeys() as $lang) {
                $value = $this->abeLOMgetClassificationDescription($lang);
                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."description");
                $e->appendChild($e_fils);

                $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."string");
                $e_fils2->setAttribute("language",$lang);
                $e_fils->appendChild($e_fils2);
                $t = $this->m_domDocument->createTextNode($value);
                $e_fils2->appendChild($t);                
            }
        }

        //9.4
        if(count($this->abeLOMgetClassificationKeywordKeys())){
            foreach ($this->abeLOMgetClassificationKeywordKeys() as $lang) {
                $value = $this->abeLOMgetClassificationKeyword($lang);
                $e_fils = $this->m_domDocument->createElement($this->m_xmlPrefixLom."keyword");
                $e->appendChild($e_fils);

                $e_fils2 = $this->m_domDocument->createElement($this->m_xmlPrefixLom."string");
                $e_fils2->setAttribute("language",$lang);
                $e_fils->appendChild($e_fils2);
                $t = $this->m_domDocument->createTextNode($value);
                $e_fils2->appendChild($t);                
            }
        }

        if(!$vide) {
            $this->m_domRootElement->appendChild($e);
            $this->m_domCurrentElement = $e;
        }

    }

    /**
      * Exporte la structure XML
      * @note fonction publique, les autres sont protected pour l'implémentation
      */
    function abeLOMExportAsXML(){
        //ABULEDU_LOG_TRACE
        $this->p_abeLOMExportAsXML_header();
        $this->p_abeLOMExportAsXML_section1();
        $this->p_abeLOMExportAsXML_section2();
        $this->p_abeLOMExportAsXML_section4();
        $this->p_abeLOMExportAsXML_section5();
        $this->p_abeLOMExportAsXML_section6();
        $this->p_abeLOMExportAsXML_section7();
        //$this->p_abeLOMExportAsXML_section8();
        $this->p_abeLOMExportAsXML_section9();   
        return $this->m_domDocument; 
    }


    //============================================================================= IMPORT XML
    /**
      * importe la section 1 Général
      *
      * @note Exemple de code XML source
      * @param e noeud XML de la branche 1: general
      * @code
   <lom:general>
      <lom:identifier>
         <lom:catalog>URI</lom:catalog>
         <lom:entry>http://mediatheque.abuledu.org/URI/5343xdfg32</lom:entry>
      </lom:identifier>
      <lom:title>
         <lom:string language="fre">Le titre de la ressource</lom:string>
      </lom:title>
      <lom:language>fre</lom:language>
      <lom:description>
         <lom:string language="fre">Résumé de la ressource.</lom:string>
      </lom:description>
      <lom:keyword>
         <lom:string language="fre">19e-20e siècle</lom:string>
      </lom:keyword>
      <lom:keyword>
         <lom:string language="fre">lutte politique, résistance : politique</lom:string>
      </lom:keyword>
      <lom:structure>
         <lom:source>LOMv1.0</lom:source>
         <lom:value>networked</lom:value>
      </lom:structure>
      <lom:aggregationLevel>
         <lom:source>LOMv1.0</lom:source>
         <lom:value>2</lom:value>
      </lom:aggregationLevel>
   </lom:general>
      * @endcode
      */
    function p_abeLOMImportXML_section1($n){
        //ABULEDU_LOG_TRACE

        foreach ($n->childNodes as $fils) {

            if($fils->nodeType == 3){
                continue;
            }

            if($fils instanceof DomElement){
                $e = $fils;

                //1.1 Identifiant
                if($e->nodeName == $this->m_xmlPrefixLom."identifier"){

                
                    foreach ($e->childNodes as $node) {
                        //catalog
                        if($node->nodeName == $this->m_xmlPrefixLom."catalog"){
                            $this->abeLOMsetGeneralIdentifierCatalog($node->textContent);
                        }

                        //entry
                        if($node->nodeName == $this->m_xmlPrefixLom."entry"){

                            $this->abeLOMsetGeneralIdentifierEntry($node->textContent);
                        }
                    }
                }

                //1.2 Titre
                if($e->nodeName == $this->m_xmlPrefixLom."title"){

                    foreach ($e->childNodes as $node) {
                        if($node->nodeType == 3){
                            continue;
                        }
                        $this->abeLOMsetGeneralTitle($node->getAttribute("language"),$node->textContent);
                    }                   
                }

                //1.3 Languages
                if($e->nodeName == $this->m_xmlPrefixLom."language"){
                    $this->abeLOMaddGeneralLanguage($e->textContent);
                }

                //1.4 Description
                if($e->nodeName == $this->m_xmlPrefixLom."description"){
                    foreach ($e->childNodes as $node) {
                        if($node->nodeType == 3){
                            continue;
                        }
                        $this->abeLOMsetGeneralDescription($node->getAttribute("language"),$node->textContent);
                    }
                }

                //1.5 Mot clés
                if($e->nodeName == $this->m_xmlPrefixLom."keyword"){
                    foreach ($e->childNodes as $node) {
                        if($node->nodeType == 3){
                            continue;
                        }
                        $this->abeLOMaddGeneralKeyword($node->getAttribute("language"),$node->textContent);
                    }                    
                }         


                //1.6 Couverture
                if($e->nodeName == $this->m_xmlPrefixLom."coverage"){
                    foreach ($e->childNodes as $node) {
                        if($node->nodeType == 3){
                            continue;
                        }                        
                        $this->abeLOMaddGeneralCoverage($node->getAttribute("language"),$node->textContent);
                    }                    
                } 

                //1.7 structure
                if($e->nodeName == $this->m_xmlPrefixLom."structure"){
                    foreach ($e->childNodes as $value) {
                        if($value->nodeType == 3)
                            continue;
                        if($value->nodeName == $this->m_xmlPrefixLom."value"){
                            $this->abeLOMsetGeneralStructure($value->textContent);
                        }   
                    }                 
                }

                //1.8 niveau d'agrégation
                 if($e->nodeName == $this->m_xmlPrefixLom."aggregationLevel"){
                    foreach ($e->childNodes as $value) {
                        if($value->nodeType == 3)
                            continue;
                        if($value->nodeName == $this->m_xmlPrefixLom."value"){
                            $this->abeLOMsetGeneralAggregationLevel($value->textContent);
                        }
                    }
                }
            }
        }

        return true;    
    }

    /**
      * Import la section 2 Cycle de vie
      * @param e noeud XML de la branche 2: lifeCycle
      * @code
    <lom:lifeCycle>
        <lom:version>
            <lom:string language="fre">Un intitulé libre de version</lom:string>
        </lom:version>
        <lom:status>
            <lom:source>LOMv1.0</lom:source>
            <lom:value>final</lom:value>
        </lom:status>
        <lom:contribute>
            <lom:role>
                <lom:source>LOMv1.0</lom:source>
                <lom:value>author</lom:value>
            </lom:role>
            <lom:entity>BEGIN:VCARD
VERSION:3.0
N:Nom; Prenom;;;
FN:Nom Prenom
ORG:Nom de l'organisme du "contributeur";;
EMAIL;INTERNET:adresse.email@domaine.fr
END:VCARD
            </lom:entity>
            <lom:date>
                <lom:dateTime>2012-04-17</lom:dateTime>
                <lom:description>
                    <lom:string language="fre">un petit commentaire</lom:string>
                </lom:description>
            </lom:date>
        </lom:contribute>
    </lom:lifeCycle>
      * @endcode
      */
    function p_abeLOMImportXML_section2($n){
        //ABULEDU_LOG_TRACE
        foreach ($n->childNodes as $fils) {

            if($fils->nodeType == 3){
                continue;
            }

            if($fils instanceof DomElement){
                $e = $fils;
                //ABULEDU_LOG_DEBUG

                //2.1 Version
                if($e->nodeName == $this->m_xmlPrefixLom."version"){
                    foreach($e->childNodes as $version){
                        if($version->nodeType != 3){
                            $this->abeLOMsetLifeCycleVersion($version->getAttribute("language"), $version->textContent);
                        }
                    }
                }

                //2.2 Status
                if($e->nodeName == $this->m_xmlPrefixLom."status"){
                    foreach($e->childNodes as $status){
                        if($status->nodeType != 3 && $status->nodeName == $this->m_xmlPrefixLom ."value"){
                            $this->abeLOMsetLifeCycleStatus($status->textContent);
                        }
                    }
                }

                //2.3 Contribution (n)
                if($e->nodeName == $this->m_xmlPrefixLom."contribute"){
                    $role;
                    $ladate;
                    $entity;
                    $entityTemp = array();

                    foreach ($e->childNodes as $ElementContrib) {
                        if($ElementContrib->nodeType == 3)
                            continue;


                        if($ElementContrib->nodeName == $this->m_xmlPrefixLom."role"){
                            foreach($ElementContrib->childNodes as $elem){
                                if($elem->nodeType != 3)
                                    if($elem->nodeName == $this->m_xmlPrefixLom."value")
                                        $role = $elem->textContent;
                                    
                            }
                        }

                        if($ElementContrib->nodeName == $this->m_xmlPrefixLom."entity"){
                            //Une liste temporaire des entites  
                            

                            $parse = File_IMC::parse('vCard');
                            $info = $parse->fromText($ElementContrib->textContent);
                            $vcard = File_IMC::build('vCard');
                            $vcard->setVersion('2.1');
                            $vcard->setFromArray($info['VCARD']);
                            array_push($entityTemp, $vcard);
                        }

                        if($ElementContrib->nodeName == $this->m_xmlPrefixLom."date"){
                            foreach ($ElementContrib->childNodes as $dateTime) {
                                if($dateTime->nodeName == $this->m_xmlPrefixLom."dateTime"){
                                    $ladate = DateTime::createFromFormat("Y-m-d", $dateTime->textContent);
                                }
                            }
                        }
                    }
                    foreach($entityTemp as $value){
                        $this->abeLOMaddLifeCycleContributionRole($role, $value, $ladate);
                    }
                }

            }

        }       
        return true;
    }

    function p_abeLOMImportXML_section3($n){
        $xmlSection3 = $n->childNodes;

        foreach ($xmlSection3 as $value) {

           
            if($value->nodeName == $this->m_xmlPrefixLom."identifier"){
                foreach ($value->childNodes as $element2) {
                    if($element2->nodeName == $this->m_xmlPrefixLom."catalog"){
                        $catalog = $element2->textContent;
                        $this->abeLOMsetMetaMetadataIdentifierCatalog($catalog);
                    }
                    if($element2->nodeName == $this->m_xmlPrefixLom."entry"){
                        $entry = $element2->textContent;
                        $this->abeLOMsetMetaMetadataIdentifierEntry($entry);
                    }       
                }
                //trim supp
                if($catalog != ""){
                    $this->abeLOMaddRelationResourceIdentifier($catalog, $entry);
                }
            }

                //2.3 Contribution (n)
                if($value->nodeName == $this->m_xmlPrefixLom."contribute"){
                    $role;
                    $ladate;
                    $entity;
                    $entityTemp = array();

                    foreach ($value->childNodes as $ElementContrib) {
                        if($ElementContrib->nodeType == 3)
                            continue;

                        if($ElementContrib->nodeName == $this->m_xmlPrefixLom."role"){
                            foreach($ElementContrib->childNodes as $elem){
                                if($elem->nodeType != 3)
                                    if($elem->nodeName == $this->m_xmlPrefixLom."value")
                                        $role = $elem->textContent;
                            }
                        }

                        if($ElementContrib->nodeName == $this->m_xmlPrefixLom."entity"){
                            //Une liste temporaire des entites
                            $parse = File_IMC::parse('vCard');
                            $info = $parse->fromText($ElementContrib->textContent);
                            $vcard = File_IMC::build('vCard');
                            $vcard->setVersion('2.1');
                            $vcard->setFromArray($info['VCARD']);
                            array_push($entityTemp, $vcard);
                        }

                        if($ElementContrib->nodeName == $this->m_xmlPrefixLom."date"){
                            foreach ($ElementContrib->childNodes as $dateTime) {
                                if($dateTime->nodeName == $this->m_xmlPrefixLom."dateTime"){
                                    $ladate = DateTime::createFromFormat("Y-m-d", $dateTime->textContent);
                                }
                            }
                        }
                    }
                    foreach($entityTemp as $value){
                        $this->abeLOMaddMetaMetadataContribute($role, $value, $ladate);
                    }
                }

            if($value->nodeName == $this->m_xmlPrefixLom."metadataSchema"){
                $this->abeLOMaddMetaMetadataMetadataSchema($value->textContent);                
            }


            if($value->nodeName == $this->m_xmlPrefixLom."language"){
                $this->abeLOMsetMetaMetadataLanguage($value->textContent);                
            }            
        }        
        return false;
    }

    function p_abeLOMImportXML_section4($n){
        //ABULEDU_LOG_TRACE
        $xmlSection4 = $n->childNodes;
        foreach ($xmlSection4 as $value) {
            //4.1 Format
            if($value->nodeName == $this->m_xmlPrefixLom."format"){
                $this->abeLOMaddTechnicalFormat($value->textContent);
            }

            //4.2 Taille 
            if($value->nodeName == $this->m_xmlPrefixLom."size"){
                $this->abeLOMsetTechnicalSize($value->textContent);
            }

            //4.3 Localisation
            if($value->nodeName == $this->m_xmlPrefixLom."location"){
                $this->abeLOMaddTechnicalLocation($value->textContent);
            }            

        }
    
    }


    function p_abeLOMImportXML_section5($n){
        //ABULEDU_LOG_TRACE
        $xmlSection5 = $n->childNodes;
        foreach ($xmlSection5 as $value) {
            //ABULEDU_LOG_DEBUG
            //5.1 type d'interactivité : uniquement dans lom ... et pas utilisé !
            if($value->nodeName == $this->m_xmlPrefixLom."interactivityType"){
                foreach ($value->childNodes as $element) {
                    if($element->nodeName == $this->m_xmlPrefixLom."value"){
                        $this->abeLOMsetEducationalInteractivityType($element->textContent);
                    }
                 }
            }  

            //5.2 type de ressource
            if($value->nodeName == $this->m_xmlPrefixLom."learningResourceType"){
                foreach ($value->childNodes as $element) {
                    if($element->nodeName == $this->m_xmlPrefixLom."value"){
                        $this->abeLOMaddEducationalLearningResourceType($element->textContent);
                    }
                 } 
            }

            //5.3 niveau d'interactivité - pas utilisé
            if($value->nodeName == $this->m_xmlPrefixLom."interactivityLevel"){
                foreach ($value->childNodes as $element) {
                    if($element->nodeName == $this->m_xmlPrefixLom."value"){
                        $this->abeLOMsetEducationalInteractivityLevel($element->textContent);
                    }
                } 
            }

            //5.4 densité sémantique - pas utilisé           
            if($value->nodeName == $this->m_xmlPrefixLom."semanticDensity"){
                foreach ($value->childNodes as $element) {
                    if($element->nodeName == $this->m_xmlPrefixLom."value"){
                        $this->abeLOMsetEducationalSemanticDensity($element->textContent);
                    }
                } 
            }

            //5.5 role de l'utilisateur final
            if($value->nodeName == $this->m_xmlPrefixLom."intendedEndUserRole"){
                foreach ($value->childNodes as $element) {
                    if($element->nodeName == $this->m_xmlPrefixLom."value"){
                        $this->abeLOMaddEducationalIntendedEndUserRole($element->textContent);
                    }
                } 
            }

            //5.6 contexte
            if($value->nodeName == $this->m_xmlPrefixLom."context"){
                foreach ($value->childNodes as $element) {
                    if($element->nodeName == $this->m_xmlPrefixLom."value"){
                        $this->abeLOMaddEducationalContext($element->textContent);
                    }
                } 
            }

            //5.7 tranche d'age
            if($value->nodeName == $this->m_xmlPrefixLom."typicalAgeRange"){
                foreach ($value->childNodes as $element) {
                    if($element->nodeName == $this->m_xmlPrefixLom."string"){
                        $this->abeLOMaddEducationalTypicalAgeRange($element->getAttribute("language"),$element->textContent);     
                    }
                } 
            }

            //5.8 difficulté
            if($value->nodeName == $this->m_xmlPrefixLom."difficulty"){
                foreach ($value->childNodes as $element) {
                    if($element->nodeName == $this->m_xmlPrefixLom."value"){
                        $this->abeLOMsetEducationalDifficulty($element->textContent);
                    }
                } 
            }

            //5.9 temps d'apprentissage moyen
            if($value->nodeName == $this->m_xmlPrefixLom."typicalLearningTime"){
                foreach ($value->childNodes as $element) {
                    if($element->nodeName == $this->m_xmlPrefixLom."value"){
                        $timeTmp = DateTime::createFromFormat("\P\TH\Hi\Ms\S", $element->textContent);
                        $this->abeLOMsetEducationalTypicalLearningTime($timeTmp->format('H')*60+$timeTmp->format('i'));
                    }
                } 
            }

            //5.10 description / proposition d'utilisation
            if($value->nodeName == $this->m_xmlPrefixLom."description"){
                foreach ($value->childNodes as $element) {
                    if($element->nodeName == $this->m_xmlPrefixLom."string"){
                        $this->abeLOMaddEducationalDescription($element->getAttribute("language"),$element->textContent);
                    }
                } 
            }

            //5.11 langue
            if($value->nodeName == $this->m_xmlPrefixLom."language"){
                $this->abeLOMaddEducationalLanguage($value->textContent);
            }           
        }
    }

    function p_abeLOMImportXML_section6($n){
        //ABULEDU_LOG_TRACE
        $xmlSection6 = $n->childNodes;

        foreach ($xmlSection6 as $value) {
            //6.1 cout
            if($value->nodeName == $this->m_xmlPrefixLom."cost"){
                foreach ($value->childNodes as $element) {
                    if($element->nodeName == $this->m_xmlPrefixLom."value"){
                        $this->abeLOMsetRightsCost($element->textContent);
                    }
                } 
            }

            //6.2 copyright
            if($value->nodeName == $this->m_xmlPrefixLom."copyrightAndOtherRestrictions"){
                foreach ($value->childNodes as $element) {
                    if($element->nodeName == $this->m_xmlPrefixLom."value"){
                        $this->abeLOMsetRightsCopyrightAndOtherRestrictions($element->textContent);
                    }
                } 
            }

            //6.3 Description
            if($value->nodeName == $this->m_xmlPrefixLom."description"){
                foreach ($value->childNodes as $element) {
                    if($element->nodeName == $this->m_xmlPrefixLom."string"){
                        $this->abeLOMsetRightsDescription($element->getAttribute("language"),$element->textContent);
                    }
                } 
            }                       
        }

        return true;   
    }

    /**
      * Import la section 7 relations
      * @param e noeud XML de la branche 7: relation
      * @code
    <lom:relation>
        <lom:kind>
            <lom:source>LOMv1.0</lom:source>
            <lom:value>ispartof</lom:value>
            <!--ispartof/haspart/isversionof/hasversion/isformatof/hasformat/isreferencedby/references/isbasedon/isbasisfor/requires/isrequiredby -->
        </lom:kind>
        <lom:resource>
            <lom:identifier>
                <lom:catalog>nom du catalogue où la ressource destination est stockée</lom:catalog>
                <lom:entry> Valeur de la ressource destination avec lequel est fait le lien. Cela
                    peut être url du document ou du chapitre ou autre moyen qui permet de retrouver
                    le document</lom:entry>
            </lom:identifier>
            <lom:description>
                <lom:string language="fre">Description de la ressource "destination"</lom:string>
            </lom:description>
        </lom:resource>
    </lom:relation>
      */
    function p_abeLOMImportXML_section7($n){
        //ABULEDU_LOG_TRACE
        $xmlSection7 = $n->childNodes;

        foreach ($xmlSection7 as $value) {

            //7.1 kind
            if($value->nodeName == $this->m_xmlPrefixLom."kind"){
                foreach ($value->childNodes as $element) {
                    if($element->nodeName == $this->m_xmlPrefixLom."value"){
                        $type = $element->textContent;
                        $this->abeLOMsetRelationKind($element->textContent);
                    }
                } 
            }

            //7.2
            if($value->nodeName == $this->m_xmlPrefixLom."resource"){
                foreach ($value->childNodes as $element) {
                    //7.2.1 identifiant
                    if($element->nodeName == $this->m_xmlPrefixLom."identifier"){
                        foreach ($element->childNodes as $element2) {
                            if($element2->nodeName == $this->m_xmlPrefixLom."catalog"){
                                $catalog = $element2->textContent;
                            }
                            if($element2->nodeName == $this->m_xmlPrefixLom."entry"){
                                $entry = $element2->textContent;
                            }       
                        }
                        //trim supp
                        if($catalog != ""){
                            $this->abeLOMaddRelationResourceIdentifier($catalog, $entry);
                        }
                    }

                    //7.2.2 Description, LomLS
                    $descriptionLomLS = new LomLS();
                    if($element->nodeName == $this->m_xmlPrefixLom."description"){
                        foreach ($element->childNodes as $element2) {
                            if($element2->nodeType == 3){
                                continue;
                            }
                            $descriptionLomLS->add($element2->getAttribute("language"),$element2->textContent);
                            $this->abeLOMaddRelationResourceDescription($element2->getAttribute("language"), $element2->textContent);
                        }
                        $this->abeLOMaddRelation($type,$catalog, $entry, $descriptionLomLS);
                    }

                }
                
            }           
        }

        return true;
    }

    /**
    <lom:annotation>
        <!-- L'usage des VCard est préconisée pour être en conformité avec la politique de modernisation de l'état-->
        <lom:entity>[BEGIN:VCARD VERSION:3.0 N:Nom; Prenom;;; FN:Nom Prenom ORG:Nom de l'organisme
            du "contributeur" EMAIL;INTERNET:Nom@NomOrganisme.fr END:VCARD]</lom:entity>
        <lom:date>
            <lom:dateTime>2008</lom:dateTime>
            <!-- Date de la contribution -->
            <!-- La date doit être présentée sous cette forme: AAAA-MM-JJ -->
            <lom:description>
                <lom:string language="fre">commentaire</lom:string>
            </lom:description>
        </lom:date>
        <lom:description>
            <lom:string language="fre">Contenu du commentaire sur l'utilisation de la
            ressource</lom:string>
        </lom:description>
    </lom:annotation>
      */
    function p_abeLOMImportXML_section8($n){
        //ABULEDU_LOG_TRACE
        $xmlSection8 = $n->childNodes;
        foreach ($xmlSection8 as $value) {
            //Entity
            if($value->nodeType == 3){
                continue;
            }
            if($value->nodeName == $this->m_xmlPrefixLom."entity"){
                $parse = File_IMC::parse('vCard');
                $info = $parse->fromText($value->textContent);
                $entity = File_IMC::build('vCard');
                $entity->setVersion('2.1');
                $entity->setFromArray($info['VCARD']);       
            }
            //Date
            if($value->nodeName == $this->m_xmlPrefixLom."date"){
                foreach ($value->childNodes as $date) {
                    if($date->nodeName == $this->m_xmlPrefixLom."dateTime"){
                        $dateAnnotation = DateTime::createFromFormat('Y-m-d', $date->textContent);
                    }
                }            
            }

            //Description
            if($value->nodeName == $this->m_xmlPrefixLom."description"){  
                $descriptionLom = new LomLS();
                foreach ($value->childNodes as $descriptionString) {
                    if($descriptionString->nodeName == $this->m_xmlPrefixLom."string"){
                        $langDescription = $descriptionString->getAttribute("language");
                        $descriptionLom->add($langDescription, $descriptionString->textContent);
                    }
                }
            } 
        }
        $this->abeLOMaddAnnotation($entity, $dateAnnotation, $descriptionLom);

        return true;
    }

    /*
    <lom:classification>
     <lom:purpose>
      <lom:source>LOMv1.0</lom:source>
      <lom:value>educational level</lom:value>
     </lom:purpose>
     <lom:taxonPath>
      <lom:source>
       <lom:string language="fre">nomenclature niveaux Sceren V 2.1</lom:string>
      </lom:source>
      <lom:taxon>
       <lom:id></lom:id>
       <lom:entry>
        <lom:string language="fre">voies générale et technologique collège cycle d'orientation 3e</lom:string>
       </lom:entry>
      </lom:taxon>
      <lom:taxon>
       <lom:id></lom:id>
       <lom:entry>
        <lom:string language="fre">voies générale et technologique lycée</lom:string>
       </lom:entry>
      </lom:taxon>
      <lom:taxon>
       <lom:id></lom:id>
       <lom:entry>
        <lom:string language="fre">voie professionnelle lycée CAP CAP Santé-Social</lom:string>
       </lom:entry>
      </lom:taxon>
     </lom:taxonPath>
    </lom:classification>
    */
    function p_abeLOMImportXML_section9($n){
        //ABULEDU_LOG_TRACE
        //$test->p_abeLOMImportXML_section9($test->abeLOMExportAsXML()->documentElement);
        $xmlSection9 = $n->childNodes;
        foreach ($xmlSection9 as $value) {
            if($value->nodeType == 3){
                continue;
            }
            $sourceTaxonPath;
            $idTaxon;
            $entryTaxon = new LomLS();
            $listeEntryTaxon = array();

            //9.1 purpose
            if($value->nodeName == $this->m_xmlPrefixLom."purpose"){
                foreach ($value->childNodes as $element) {
                    if($element->nodeName == $this->m_xmlPrefixLom."value"){
                        $objectif = $element->textContent;
                    }
                    if($element->nodeName == $this->m_xmlPrefixLom."source"){
                        //rien
                    }
                }
            }


            //9.2 taxonPath
            if($value->nodeName == $this->m_xmlPrefixLom."taxonPath"){ 
                //9.2.1
                foreach ($value->childNodes as $element) {

                    if($element->nodeName == $this->m_xmlPrefixLom."source"){
                        foreach ($element->childNodes as $element2) {
                            if($element2->nodeType == 3)
                                continue;
                            $sourceTaxonPath = new LomLS($element2->getAttribute("language"), $element2->textContent);
                        }
                    }
                }

                foreach ($value->childNodes as $element){
                    if($element->nodeName == $this->m_xmlPrefixLom."taxon"){

                        foreach ($element->childNodes as $elementTaxon) {
                            //id taxon
                            if($elementTaxon->nodeName == $this->m_xmlPrefixLom."id"){
                                $idTaxon = $elementTaxon->textContent;
                            }
                            //entry taxon
                            if($elementTaxon->nodeName == $this->m_xmlPrefixLom."entry"){
                                foreach ($elementTaxon->childNodes as $element2) {
                                    if($element2->nodeType == 3)
                                        continue;
                                    $entryTaxon = new LomLS($element2->getAttribute("language"),$element2->textContent);
                                    array_push($listeEntryTaxon, $entryTaxon);

                                }    
                            }
                            
                        }
                        
                        $this->abeLOMaddClassificationPurpose($objectif, $sourceTaxonPath, $idTaxon, $entryTaxon);
                    }
                }

                //$this->abeLOMaddClassificationPurpose($objectif, $sourceTaxonPath, $idTaxon, $listeEntryTaxon);

            }

            //9.3 description
            if($value->nodeName == $this->m_xmlPrefixLom."description"){ 
                foreach ($value->childNodes as $child) {
                    if($child->nodeType !== 3)
                        $this->abeLOMsetClassificationDescription($child->getAttribute("language"), $child->textContent);
                }
               
            }

            //9.3 keyword
            if($value->nodeName == $this->m_xmlPrefixLom."keyword"){ 
                foreach ($value->childNodes as $child) {
                    if($child->nodeType !== 3)
                        $this->abeLOMsetClassificationKeyword($child->getAttribute("language"), $child->textContent);
                }
            }
        }

        return true;       
    }

    /** Ouvre le fichier fileName et affecte son contenu a la m_domDocument */
    function abeLOMsetFile($fileName){
        //ABULEDU_LOG_TRACE(spe)
        $f = fopen($fileName, "r+");
        if($f==false)
            die("unable to create file");
        return $this->abeLOMsetXML($fileName);

    }

    /** affecte directement du XML a la m_domDocument */
    function abeLOMsetXML($xml){
        //ABULEDU_LOG_TRACE
        $this->clear();


        if(!$this->m_domDocument->load($xml)){
            //ABULEDU_LOG_ERROR
            echo "chargement XML impossible  (Message à supprimer)";
            return false;
        }


        //Ensuite il faut "parcourir" le fichier XML
        //print out the element names of all elements that are direct children
        //of the outermost element.
        $numSection9 = 0;
        foreach ($this->m_domDocument->documentElement->childNodes as $element) {
            if($element->nodeType != 3){
                if($element->nodeName == $this->m_xmlPrefixLom."general"){

                    $this->p_abeLOMImportXML_section1($element);
                }

                if($element->nodeName == $this->m_xmlPrefixLom."lifeCycle"){
                    $this->p_abeLOMImportXML_section2($element);
                }
                if($element->nodeName == $this->m_xmlPrefixLom."metaMetadata"){
                    $this->p_abeLOMImportXML_section3($element);
                }
                if($element->nodeName == $this->m_xmlPrefixLom."technical"){
                    $this->p_abeLOMImportXML_section4($element);
                }
                if($element->nodeName == $this->m_xmlPrefixLom."educational"){
                    $this->p_abeLOMImportXML_section5($element);
                }
                if($element->nodeName == $this->m_xmlPrefixLom."rights"){
                    $this->p_abeLOMImportXML_section6($element);
                }
                if($element->nodeName == $this->m_xmlPrefixLom."relation"){
                    $this->p_abeLOMImportXML_section7($element);
                }
                if($element->nodeName == $this->m_xmlPrefixLom."annotation"){
                    $this->p_abeLOMImportXML_section8($element);
                }
                if($element->nodeName == $this->m_xmlPrefixLom."classification"){
                    $this->p_abeLOMImportXML_section9($element);
                }
            }
        }
        return true;
    }



    /** Retourne une liste de vcard : toutes les vcard rencontrées dans le fichier XML */
    function abeLOMgetAllVcards(){
        //ABULEDU_LOG_TRACE
        $empty = File_IMC::build('vCard');
        $empty->setFormattedName("Non renseigné");

        $liste = array();
        if(count($this->m_allVcards) == 0){
            array_push($liste, $empty);
            return $liste;
        }

        return $this->m_allVcards;
    }

    /** Retourne la vCard correspondant au nom fullName */
    function abeLOMgetVCardFromFullName($fullName){
        //ABULEDU_LOG_TRACE
        foreach ($this->m_allVcards as $vcard) {
            if($vcard->get("FN") == $fullName){
                return $vcard;
            }
        }

        return File_IMC::build('vCard');
    }

    /** Passe la classe en mode debug ou pas */
    //void abeLOMsetLocalDebug(int level);



protected $m_LifeCycleContributions;
protected $m_availableLanguages;
protected $m_allVcards;
protected $m_GeneralIdentifierCatalog;
protected $m_GeneralIdentifierEntry;
protected $m_GeneralTitle;
protected $m_GeneralLanguage;
protected $m_GeneralDescription;
protected $m_GeneralKeyword;
protected $m_GeneralCoverage;
protected $m_GeneralStructure;
protected $m_GeneralStructureVocabLOM;
protected $m_GeneralAggregationLevel;
protected $m_GeneralAggregationLevelVocabLOM;
protected $m_LifeCycleVersion;
protected $m_LifeCycleStatus;
protected $m_LifeCycleStatusVocabLOM;
protected $m_LifeCycleContributionRoleVocabLOM;
protected $m_MetaMetadata;
protected $m_MetaMetadataContribute;
protected $m_MetaMetadataIdentifierCatalog;
protected $m_MetaMetadataIdentifierEntry;
protected $m_MetaMetadataContributeRole;
protected $m_MetaMetadataContributeEntity;
protected $m_MetaMetadataContributeDate;
protected $m_MetaMetadataMetadataSchema;
protected $m_MetaMetadataLanguage;
protected $m_TechnicalFormat;
protected $m_TechnicalSize;
protected $m_TechnicalLocation;
protected $m_TechnicalRequirementOrCompositeType;
protected $m_TechnicalRequirementOrCompositeName;
protected $m_TechnicalRequirementOrCompositeMinimumVersion;
protected $m_TechnicalRequirementOrCompositeMaximumVersion;
protected $m_TechnicalInstallationRemarks;
protected $m_TechnicalOtherPlatformRequirements;
protected $m_TechnicalDuration;
protected $m_EducationalInteractivityType;
protected $m_EducationalInteractivityTypeVocabLOM;
protected $m_EducationalLearningResourceType;
protected $m_EducationalLearningResourceTypeVocabLOM;
protected $m_EducationalInteractivityLevel;
protected $m_EducationalSemanticDensity;
protected $m_EducationalIntendedEndUserRole;
protected $m_EducationalIntendedEndUserRoleVocabLOM;
protected $m_EducationalContext;
protected $m_EducationalContextVocabLOM;
protected $m_EducationalTypicalAgeRange;
protected $m_EducationalDifficulty;
protected $m_EducationalDifficultyVocabLOM;
protected $m_EducationalTypicalLearningTime;
protected $m_EducationalDescription;
protected $m_EducationalLanguage;
protected $m_RightsCost;
protected $m_RightsCostVocabLOM;
protected $m_RightsCopyrightAndOtherRestrictions;
protected $m_RightsCopyrightAndOtherRestrictionsVocabLOM;
protected $m_RightsDescription;
protected $m_Relation;

//Variable non utilisés (cardinalité n non pris en charge)
protected $m_RelationKind;
protected $m_RelationKindVocabLOM;
protected $m_RelationResourceIdentifierEntry;
protected $m_RelationResourceDescription;
//fin variable inutilisés


protected $m_Annotation;

//Variable non utilisés (cardinalité n non prise en charge)
protected $m_AnnotationEntity;
protected $m_AnnotationDate;
protected $m_AnnotationDescription;
//fin varible non utilisés

protected $m_ClassificationPurpose;
protected $m_ClassificationPurposeVocabLOM;
protected $m_ClassificationTaxonPathSource;
protected $m_ClassificationTaxonPathTaxumID;
protected $m_ClassificationTaxonPathTaxumEntry;
protected $m_classificationPurposes;
protected $m_ClassificationDescription;
protected $m_ClassificationKeyword;
protected $m_domDocument;
protected $m_domRootElement;
protected $m_domCurrentElement;
protected $m_xmlPrefixLom;
}
?>